#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Flask.WebApi 应用程序 ]

"""

from flask import Flask
from flask.blueprints import Blueprint

from rains.baseic.log import LOG
from rains.baseic.config import CONFIG
from rains.baseic.decorator import singleton_pattern

from rains.server.api.blueprint import __all__ as blueprint_all


# ----------------------------------
@singleton_pattern
class RainsApp(object):
    """
    [ RainsApi 应用程序 ]

    * 基于 Flask 实现的 Flask.WebApi 应用程序.

    """

    _app_living: Flask = Flask(__name__)
    """ [ 应用程序实例 ] """

    # ------------------------------
    def __init__(self):
        """ 
        [ RainsApi 应用程序 ]
        
        """

        self.update_config({'JSON_AS_ASCII': False})

        for blueprint in blueprint_all:
            self.load_blueprint(blueprint)

    # ------------------------------
    @property
    def living(self) -> Flask:
        """
        [ 应用程序实例 ]

        * 返回应用程序实例.

        ---
        返回:
            Flask : Flask 应用程序实例.

        """

        return self._app_living

    # ------------------------------
    def update_config(self, config_dict: dict) -> None:
        """
        [ 更新配置项 ]

        * 更新 Flask 应用程序的配置项.

        ---
        参数:
            config { dict } : 配置字典.

        """

        self._app_living.config.update(config_dict)

        if CONFIG.debug:
            for ck, cv in config_dict.items():
                LOG.info(f'{ self.__class__.__name__ } :: 更新配置 :: [{ ck } = { cv }]')

    # ------------------------------
    def load_blueprint(self, blueprint: Blueprint):
        """
        [ 载入蓝图 ]

        * 将蓝图对象载入 Flask 应用程序.

        ---
        参数:
            blueprint { Blueprint } : Flask.Blueprint 蓝图对象.

        """

        self._app_living.register_blueprint(blueprint)

        if CONFIG.debug:
            LOG.info(f'{ self.__class__.__name__ } :: 载入蓝图 :: [{ blueprint }]')



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 运行服务器 ]

"""

import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))

from rains.baseic.const import CONST


def function_run_server():
    """
    [ 运行服务器 ]
    
    """

    if not CONST.SYS.PATH_RAINS_INI_FILE.is_file():
        print(_MESSAGE_ERROR)
        exit(0)

    port: int = 3700
    if len(sys.argv) >= 3:
        try:
            port = int(sys.argv[2])
        except ValueError:
            print('运行服务器指令的端口参数必须为数字!')
            exit(0)

    print(_MESSAGE)

    from rains.core.perform_server import PerformServer
    perform_server = PerformServer()
    perform_server.running(port)


_MESSAGE = \
"""
--------------------------------------

服务器正在运行

--------------------------------------
"""

_MESSAGE_ERROR = \
"""
--------------------------------------

请在工程根目录执行 -server 命令

初始化工程将创建或补全项目执行依赖文件, 并在工程根目录中创建 <rains.ini> 文件。

<rains.ini> 文件 仅作为工程根目录标识之用, -run|-server 命令需要在工程根目录中才允许执行。

--------------------------------------
"""




import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from rains.kit.web import *
from rains.kit.api import ApiTask


class ApiTest(ApiTask):
    """
    测试任务
    """

    url: str = 'http://127.0.0.1:5000'


    def case_01(self):
        """
        [ GET1 ]

        """

        r = self.plant.get(f'{ self.url }/get1')
        return r.data['code'] == 0 and r.data['data'] == 'get1'


    def case_02(self):
        """
        [ GET2 ]

        """

        r = self.plant.get(f'{ self.url }/get2')
        return r.data['code'] == 0 and r.data['data'] == 'get2'


    def case_03(self):
        """
        [ POST1 ]

        """

        data = {'v1': 1, 'v2': 2}
        r = self.plant.post(f'{ self.url }/post1', data)
        return r.data['code'] == 0 and r.data['data']['v2'] == '2'


    def case_04(self):
        """
        [ POST2 ]

        """

        data = {'v1': 1, 'v2': 2}
        self.plant.set_token('123123')
        r = self.plant.post(f'{ self.url }/post2', data)
        return r.data['data']['v2'] == '2' and r.data['token'] == '123123'


class TASK:
    def __init__(self) -> None:
        print("123456")


class TaskTest(WebTask):
    """
    测试任务
    """

    def set_task_init(self):
        
        print('set_task_init')

        self.user_input = self._plant.element(
            page='超管后台', 
            name='邮箱输入框', 
            by_key=BY.XPATH, 
            by_value='/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
        )

        self.password_input = self._plant.element(
            page='超管后台', 
            name='密码输入框', 
            by_key=BY.XPATH, 
            by_value='/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
        )

        self.login_button = self._plant.element(
            page='超管后台', 
            name='Login按钮', 
            by_key=BY.XPATH, 
            by_value='/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
        )

        self.create_button = self._plant.element(
            page='超管后台', 
            name='Login按钮', 
            by_key=BY.XPATH, 
            by_value='/html/body/div/div/div/div/div/div[1]/a'
        )

    def set_task_quit(self):
        self._plant.view.get.url()
        print('set_task_quit')

    def set_case_init(self):
        print('set_case_init')
        self._plant.view.page.goto('http://manage.dev-tea.cblink.net/')

    def set_case_quit(self):
        print('set_case_quit')

    def case_1(self):
        """
        登录成功
        """
        self._plant.view.get.url()
        self.user_input.input.send('admin@baocai.us')
        self.password_input.input.send('123456')
        self.login_button.mouse.tap()
        return self.create_button.get.exist() is True

    def case_2(self):
        """
        登录成功
        """
        return self.create_button.get.exist() is True


task_dict = {
    'task_name': 'TestJsonTask', 
    'task_type': 'WEB',
    'task_remark': '测试JSON任务', 
    'instructions_setter': {}, 
    'instructions_case': {
        'case_1': {
            'case_remark': '测试JSON用例1', 
            'assert': {5: True}, 
            'step_list': {
                1: {
                    'module': 'view', 
                    'class': 'page', 
                    'function': 'goto', 
                    'parameter': {
                        'url': 'http://manage.dev-tea.cblink.net/'
                    }
                }, 
                2: {
                    'module': 'element', 
                    'class': 'input', 
                    'function': 'send', 
                    'element': {
                        'page': '超管后台', 
                        'name': '邮箱输入框', 
                        'by_key': 'xpath', 
                        'by_value': '/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
                    }, 
                    'parameter': {
                        'key': 'admin@baocai.us'
                    }
                }, 
                3: {
                    'module': 'element', 
                    'class': 'input', 
                    'function': 'send', 
                    'element': {
                        'page': '超管后台', 
                        'name': '密码输入框', 
                        'by_key': 'xpath', 
                        'by_value': '/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
                    }, 
                    'parameter': {
                        'key': '123456'
                    }
                }, 
                4: {
                    'module': 'element', 
                    'class': 'mouse', 
                    'function': 'tap', 
                    'element': {
                        'page': '超管后台', 
                        'name': 'Login按钮', 
                        'by_key': 'xpath', 
                        'by_value': '/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
                    }
                }, 
                5: {
                    'module': 'element', 
                    'class': 'get', 
                    'function': 'exist', 
                    'element': {
                        'page': '超管后台', 
                        'name': 'Login按钮', 
                        'by_key': 'xpath', 
                        'by_value': '/html/body/div/div/div/div/div/div[1]/a'
                    }
                }
            }
        }
    }
}


if __name__ == '__main__':
    # rains.core.rains_server import RainsServer
    # rains_server = RainsServer()
    # rains_server.running()

    # from rains.core.rains_pool import RainsPool
    # rains_pool = RainsPool(debug=True)

    # for _i in range(1):
    #     rains_pool.put_core(WebDriver())

    # for _i in range(1):
    #     rains_pool.put_task(TaskTest)
    #     rains_pool.put_task(task_dict)

    # rains_pool.running()

    # from rains.kit.web import WebDriver
    # core = WebDriver()
    # core.start_task(TaskTest)
    # core.end()

    from rains.core.perform_pool import PerFormPool
    from rains.kit.web import WebDriver

    # with WebDriver() as driver:
    #     driver.start_task(task_dict)
    #     # driver.start_task(task_dict)

    pool = PerFormPool()
    pool.put_task(task_dict)
    pool.put_task(TaskTest)
    pool.running()



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.DB.Common ]

"""


# ----------------------------------
class RainsDbParameterHandler(object):
    """
    [ 数据库参数处理程序 ]

    * 该类用于解析前端请求参数, 以及拼接返回给前端的 Json 数据结构.

    """

    # ------------------------------
    @staticmethod
    def get_desc_limit(page: int = 1, number: int = 10) -> str:
        """
        [ 获取数据返回量区间限制 ]

        * 默认是获取 1 页, 10 条数据.
        
        ---
        参数:
            page { int } : 页数.
            number { int } : 单页数据量.

        ---
        返回:
            Response : 拼接完成的 SQL LIMIT 语句参数.

        """
        
        # 计算偏移量
        limit_begin = 0
        if page > 1:
            limit_begin = (page - 1) * number

        return f'{ limit_begin }, { number }'



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Kit.Api.Packaging.ApiPlantRequest ]

"""

from typing import Any
from typing import Union

import requests
from flask import Response

from rains.baseic.log import LOG
from rains.baseic.config import CONFIG
from rains.baseic.tools import Regular
from rains.core.common.action_recorder import ACTION_RECORDER


# ----------------------------------
class ApiPlantResponse(object):
    """
    [ Api 响应对象 ]
    
    """

    _base_response: Response

    # ------------------------------
    def __init__(self, response: Response):
        """
        [ Api 响应对象 ]
        
        """

        self._base_response = response

    # ------------------------------
    @property
    def ok(self) -> Union[dict, None]:
        """
        [ 数据 ]
        
        """
        
        return self._base_response.ok

    # ------------------------------
    @property
    def data(self) -> Union[dict, None]:
        """
        [ 数据 ]
        
        """
        
        if self._base_response.ok:
            try:
                return self._base_response.json()
            except BaseException:
                return self._base_response.text


# ----------------------------------
class ApiPlantRequest(object):
    """
    [ Api请求器 ]

    """

    _api_driver: Any
    """ [ 引擎对象 ] """

    # ------------------------------
    def __init__(self, driver: Any):
        """
        [ Api请求器 ]

        ---
        参数:
            driver { WebEngine } : Web引擎对象.

        """

        self._api_driver = driver

    # ------------------------------
    @property
    def send(self):
        """
        [ Web视图'发送'相关函数 ]

        """

        return _ApiPlantHandlerMapGet(self._api_driver)

    # ------------------------------
    def _map_send(self):
        return self.send


# ----------------------------------
class _ApiPlantRequestMapBase(object):
    """
    [ Api请求器映射基础类 ]

    """

    _handler: Any
    _driver_id: int

    # ------------------------------
    def __init__(self, api_driver: Any):
        """
        [ Api请求器映射基础类 ]

        ---
        参数:
            api_driver { ApiDriver } : Web引擎对象.

        """

        self._handler = api_driver
        self._driver_id = api_driver.driver_id


# ----------------------------------
class _ApiPlantHandlerMapGet(_ApiPlantRequestMapBase):
    """
    [ Api请求器'发送'相关函数 ]

    """

    # ------------------------------
    def get(self, url: str) -> ApiPlantResponse:
        """
        [ GET ]

        ---
        参数:
            url { URL } : URL.

        """

        try:
            message = Regular.get_function_doc_name(self.get.__doc__)
            base_requests = requests.get(url, headers=self._handler.headers)
            execution_step_record(self._driver_id, message, url)
            return ApiPlantResponse(base_requests)

        except BaseException as error:
            raise ApiPlantRequestFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def post(self, url: str, data: dict) -> ApiPlantResponse:
        """
        [ POST ]

        ---
        参数:
            url { URL } : URL.
            data { dict } : 请求数据.

        """

        try:
            message = Regular.get_function_doc_name(self.post.__doc__)
            base_requests = requests.post(url, data=data, headers=self._handler.headers)
            execution_step_record(self._driver_id, message, url)
            return ApiPlantResponse(base_requests)

        except BaseException as error:
            raise ApiPlantRequestFunctionRunError(self._driver_id, message, error)


# ----------------------------------
def execution_step_record(driver_id: int, message: str, join_message: str = None, level: str = 'info'):
    """
    [ 执行步骤记录 ]

    * 执行时, 将同步将信息传递给日志服务与用例记录器.

    ---
    参数:
        driver_id { int } : 核心ID.
        message { str } : 记录信息.
        join_message { str } : 拼接信息.

    """

    if join_message:
        message = ''.join([message, ' >> ', f'[{ str(join_message) }]'])

    ACTION_RECORDER.write(driver_id, message)

    if CONFIG.debug:
        LOG.map(level, f'Driver[{ driver_id }] >> { message }')
        

# ----------------------------------
class ApiPlantRequestFunctionRunError(Exception):
    """ [ Api请求器函数执行异常 ] """
    
    def __init__(self, driver_id: int, message: str, error: Any):
        self.message = f'error :: { message } :: { error } !'
        self.err_name = Regular.get_function_doc_name(self.__doc__)
        execution_step_record(driver_id, message, level='error')

    def __str__(self) -> str:
        return ''.join([self.err_name, ' :: ', self.message])



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Kit.Api.ApiPlant ]

"""

from typing import Any

from rains.kit.api.packaging.api_plant_handler import ApiPlantHandler
from rains.kit.api.packaging.api_plant_request import ApiPlantRequest


# ----------------------------------
class ApiPlant(object):
    """
    [ Api 工厂构建类 ]

    """
    
    _api_driver: Any
    """ [ Api 驱动程序 ] """
    
    # ------------------------------
    def __init__(self, api_driver: Any):
        """
        [ Api 工厂构建类 ]

        ---
        参数:
            web_driver { WebDriver } : Web 驱动程序.

        """

        self._api_driver = api_driver
        
    # ------------------------------
    @property
    def driver(self):
        return self._api_driver

    # ------------------------------
    @property
    def handler(self):
        """
        [ Api 处理器 ]
        
        """

        return ApiPlantHandler(self._api_driver)

    # ------------------------------
    def _map_handler(self):
        return self.handler

    # ------------------------------
    @property
    def request(self):
        """
        [ Api 请求器 ]
        
        """

        return ApiPlantRequest(self._api_driver)

    # ------------------------------
    def _map_request(self):
        return self.request



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.DB.RainsDB ]

"""


import sqlite3
from threading import Lock

from rains.baseic.const import CONST
from rains.baseic.keyword import KEYWORD
from rains.baseic.decorator import singleton_pattern
from rains.baseic.error import RainsDbWriteError
from rains.baseic.error import RainsDbInitError
from rains.baseic.error import RainsDbReadError


# ----------------------------------
@singleton_pattern
class RainsDb(object):
    """
    [ Rains 数据库 ]

    * 基于 SQLite 数据库.

    """

    _conn: sqlite3.Connection = None
    """ [ 数据库连接对象 ] """

    _lock: Lock = Lock()
    """ [ 数据库互斥锁 ] """

    # ------------------------------
    def __init__(self):
        """
        [ Rains 数据库 ]

        """

        try:
            # 创建/连接数据库
            if not self._conn:
                self.connect()
                self._conn.commit()

        except BaseException as error:
            raise RainsDbInitError(error)

    # ------------------------------
    def read(self, sql: str) -> list:
        """
        [ 无锁读取 ]

        * 执行查询语句, 该函数不会持有数据库锁.

        ---
        参数:
            sql { str } : SQLite 查询语句.

        ---
        异常:
            RainsDbReadError : Rains 数据库读取异常.

        ---
        返回:
            list : 查询结果列表.

        """

        try:
            # 创建数据库游标
            cur = self._conn.cursor()
            # 执行查询语句
            cur.execute(sql)
            # 获取查询结果
            data = cur.fetchall()
            # 注销数据库游标
            cur.close()

            return data

        except BaseException as error:
            raise RainsDbReadError(error)

    # ------------------------------
    def lock_read(self, sql: str) -> list:
        """
        [ 持锁读取 ]

        * 执行查询语句, 该函数会持有数据库锁, 结束时释放. 

        ---
        参数:
            sql { str } : SQLite 查询语句.

        ---
        异常:
            RainsDbReadError : Rains 数据库读取异常.

        ---
        返回:
            list : 查询结果列表.

        """

        # 获取锁
        with self._lock:

            # 执行读取函数
            return self.read(sql)

    # ------------------------------
    def write(self, sql: str) -> int:
        """
        [ 无锁写入 ]

        * 执行写入语句, 该函数不会持有数据库锁.

        ---
        参数:
            sql { str } : SQLite 查询语句.

        ---
        异常:
            RainsDbWriteError : Rains 数据库写入异常.

        ---
        返回:
            int : 写入数据后返回该条数据的自增ID.

        """

        try:
            # 创建数据库游标
            cur = self._conn.cursor()
            # 执行查询语句
            cur.execute(sql)
            # 获取数据自增ID
            aid = cur.lastrowid
            # 注销数据库游标
            cur.close()

            return aid

        except BaseException as error:
            raise RainsDbWriteError(error)

    # ------------------------------
    def lock_write(self, sql: str) -> int:
        """
        [ 持锁写入 ]

        * 执行写入语句, 该函数会持有数据库锁, 结束时释放.

        ---
        参数:
            sql { str } : SQLite 查询语句.

        ---
        异常:
            RainsDbWriteError : Rains 数据库写入异常.

        ---
        返回:
            int : 写入数据后返回该条数据的自增ID.

        """

        # 获取锁
        with self._lock:

            # 执行写入函数
            return self.write(sql)

    # ------------------------------  
    def connect(self):
        """
        [ 数据库连接 ]

        """

        # 创建数据库连接对象
        self._conn = sqlite3.connect(CONST.SYS.PATH_DB_FILE, check_same_thread=False)

        # 创建数据库游标
        Cur = self._conn.cursor()

        # 构建数据库表
        for key, value in CreateTableSql.__dict__.items():
            if '__' in key:
                continue
            Cur.execute(value)

        # 注销数据库游标
        Cur.close()

    # ------------------------------
    def commit(self):
        """
        [ 事务保存 ]

        """

        self._conn.commit()

    # ------------------------------
    def rollback(self):
        """
        [ 事务回滚 ]

        """

        self._conn.rollback()

    # ------------------------------
    def quit(self):
        """
        [ 关闭数据库连接 ]

        """

        self._conn.close()


# ----------------------------------
class CreateTableSql(object):
    """
    [ 建表语句 ]

    """

    TASKS = f""" 

    CREATE TABLE IF NOT EXISTS { KEYWORD.DB.TASKS_TABLE }
    (
        { KEYWORD.ARG.TID }           INTEGER PRIMARY KEY, 
        { KEYWORD.ARG.NAME }          TEXT NOT NULL, 
        { KEYWORD.ARG.TYPE }          TEXT NOT NULL, 
        { KEYWORD.ARG.REMARK }        TEXT NOT NULL, 
        { KEYWORD.ARG.CREATED_DATE }  DATE NOT NULL, 
        { KEYWORD.ARG.STATE }         TEXT NOT NULL, 
        { KEYWORD.ARG.START_TIME }    DATE, 
        { KEYWORD.ARG.END_TIME }      DATE, 
        { KEYWORD.ARG.SPEND_TIME_S }  INT, 
        { KEYWORD.ARG.CASE_ALL }      INT, 
        { KEYWORD.ARG.CASE_PASS }     INT, 
        { KEYWORD.ARG.CASE_FAIL }     INT
    );

    """

    """
    [ 建表语句 :: 任务表 ]

    * 记录任务的执行信息.

    ---
    结构体:
        tid { int } : 任务编号, 自增的表主键.
        name { str } : 任务名称.
        remark { str } : 任务备注信息.
        created_date { date } : 创建日期.
        state { bool } : 任务状态.
        start_time { date } : 开始时间.
        end_time { date } : 结束时间.
        spend_time_s { date } : 消耗时间(秒).
        case_all { int } : 所有用例计数.
        case_pass { int } : 成功的用例计数.
        case_fail { int } : 失败用例数计数.

    """

    # ------------------------------
    CASES = f"""

    CREATE TABLE IF NOT EXISTS { KEYWORD.DB.CASES_TABLE }
    (
        { KEYWORD.ARG.CID }           INTEGER PRIMARY KEY, 
        { KEYWORD.ARG.TID }           INT NOT NULL, 
        { KEYWORD.ARG.NAME }          TEXT NOT NULL, 
        { KEYWORD.ARG.REMARK }        TEXT NOT NULL, 
        { KEYWORD.ARG.STATE }         TEXT NOT NULL, 
        { KEYWORD.ARG.CREATED_DATE }  DATE NOT NULL, 
        { KEYWORD.ARG.START_TIME }    DATE, 
        { KEYWORD.ARG.END_TIME }      DATE, 
        { KEYWORD.ARG.SPEND_TIME_S }  INTEGER, 
        { KEYWORD.ARG.RUN_STEP }      TEXT
    );
        
    """

    """
    [ 建表语句 :: 用例表 ]

    * 记录用例的执行信息.

    ---
    结构体:
        cid { int } : 用例编号, 自增的表主键.
        tid { int } : 所属的任务记录编号.
        name { str } : 用例名称.
        remark { str } : 用例备注信息.
        state { bool } : 任务状态.
        created_date { date } : 创建日期.
        start_time { date } : 开始时间.
        end_time { date } : 结束时间.
        spend_time_s { date } : 消耗时间(秒).
        run_step { str } : 运行步骤.
            
    """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 日志模块 ]

"""

import time
import logging

from rains.baseic.const import CONST
from rains.baseic.error import CreationLogLivingError
from rains.baseic.decorator import singleton_pattern


# 日志模块配置参数
_LOGGING_LEVEL = 'DEBUG'
""" [ 记录器日志级别 ] """

_BASE_HANDLE_LEVEL = 'INFO'
""" [ 默认流处理器的日志级别 ] """

_FILE_HANDLE_LEVELS = ['DEBUG', 'INFO', 'ERROR', 'WARNING']
""" [ 文件流处理器的日志级别 ] """

_LOG_OUTPUT_STRUCTURE = logging.Formatter('[%(asctime)s] [%(levelname)7s] %(message)s')
""" [ 日志输出格式对象 ] """


# ----------------------------------
class LogMapLevel(object):
    """ [ 日志映射等级 ] """
    
    debug: str = 'debug'
    info: str = 'info'
    warning: str = 'warning'
    error: str = 'error'
    critical: str = 'critical'


# ----------------------------------
@singleton_pattern
class Log(object):
    """
    [ 日志模块 ]

    """

    _logger: logging.Logger
    """ [ 记录器对象 ] """

    # ------------------------------
    def __init__(self):
        """
        [ 日志模块 ]
        
        ---
        异常:
            CreationLogLivingError : 创建日志实例异常.

        """

        try:
            self._logger = logging.getLogger()
            self._logger.setLevel(_LOGGING_LEVEL)
            self._logger.handlers.clear()
            
            self._creation_base_handle()
            self._creation_file_handle()

        except BaseException as error:
            raise CreationLogLivingError(error)

    # ------------------------------
    def map(self, level: str, message: str):
        """ 
        [ 映射日志 ]

        根据传递的 level 映射对应等级日志打印。

        ---
        示例:
            >>> from rains.baseic.log import LOG
            >>> from rains.baseic.log import LogMapLevel
            >>> LOG.map(level=LogMapLevel.error, message='error')
            :: 输出 error 日志, 等价于 LOG.error('error') .
        
        ---
        参数:
            level { str } : 日志等级.
            message { str } : 输出的日志信息.

        """

        type(self).__dict__[level](self, message)

    # ------------------------------
    def debug(self, message: str):
        """
        [ 记录 DEBUG ]

        ---
        参数:
            message { str } : 输出的日志信息.

        """

        self._logger.debug(message)

    # ------------------------------
    def info(self, message):
        """
        [ 记录 INFO ]
        
        ---
        参数:
            message { str } : 输出的日志信息.

        """

        self._logger.info(message)

    # ------------------------------
    def warning(self, message):
        """
        [ 记录 WARNING ]

        ---
        参数:
            message { str } : 输出的日志信息.

        """

        self._logger.warning(message)

    # ------------------------------
    def error(self, message):
        """
        [ 记录 ERROR ]

        ---
        参数:
            message { str } : 输出的日志信息.

        """

        self._logger.error(message)

    # ------------------------------
    def critical(self, message):
        """
        [ 记录 CRITICAL ]

        ---
        参数:
            message { str } : 输出的日志信息.

        """

        self._logger.critical(message)

    # ------------------------------
    def _creation_base_handle(self):
        """
        [ 创建默认流处理器 ]

        """

        # 创建处理器
        base_handle = logging.StreamHandler()
        # 配置日志输出格式
        base_handle.setFormatter(_LOG_OUTPUT_STRUCTURE)
        # 设置处理器日志等级
        base_handle.setLevel(_BASE_HANDLE_LEVEL)
        # 注册处理器
        self._logger.addHandler(base_handle)

    # ------------------------------
    def _creation_file_handle(self):
        """
        [ 创建文件流处理器 ]
        
        """
        
        # 创建当前项目的当天日志存放目录
        date = time.strftime('%Y-%m-%d', time.localtime())
        path = CONST.SYS.PATH_LOGS_DIR.joinpath(date)
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        # 创建文件处理器
        for v in _FILE_HANDLE_LEVELS:

            # 创建处理器
            file_handle = logging.FileHandler(f'{path.joinpath(f"{ v }.log")}')
            # 配置日志输出格式
            file_handle.setFormatter(_LOG_OUTPUT_STRUCTURE)
            # 设置处理器的日志等级
            file_handle.setLevel(v)
            # 注册处理器
            self._logger.addHandler(file_handle)


LOG: Log = Log()
""" [ 日志模块实例 ] """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 创建环境 ]

"""

from rains.baseic.const import CONST
from rains.baseic.keyword import KEYWORD
from rains.baseic.tools import YamlHandler


# 默认配置字典
BASE_CONFIGURATION_DICT = {
    KEYWORD.CONFIG.DEBUG: True, 
    KEYWORD.CONFIG.DRIVER_MAX_COUNT: 3, 
    KEYWORD.CONFIG.TASK_MAX_COUNT: 50
}


# ----------------------------------
def create_runtime_environment():
    """
    [ 创建运行环境 ] 
    
    * 创建项目运行所依赖的日志目录、数据目录、数据库文件以及配置文件.
    
    """

    # 创建项目目录
    CONST.SYS.PATH_DATA_DIR.mkdir(parents=True, exist_ok=True)
    CONST.SYS.PATH_LOGS_DIR.mkdir(parents=True, exist_ok=True)
    CONST.SYS.PATH_DB_FILE.touch(mode=0o777)

    # 如果不存在配置文件, 则创建默认配置文件
    if not CONST.SYS.PATH_CONFIG_FILE.exists():
        CONST.SYS.PATH_CONFIG_FILE.touch(mode=0o777)
        YamlHandler().write(str(CONST.SYS.PATH_CONFIG_FILE), BASE_CONFIGURATION_DICT)


# ----------------------------------
def init_engineering() -> bool:
    """ 
    [ 初始化工程 ] 

    ---
    返回:
        bool : 是否初始化成功凭证.
    
    """

    if CONST.SYS.PATH_RAINS_INI_FILE.is_file():
        return False

    else:
        CONST.SYS.PATH_RAINS_INI_FILE.touch(mode=0o777)
        return True


# ----------------------------------
def create_project(name: str):
    """ 
    [ 创建项目 ]

    ---
    参数:
        name { str } : 项目名称.

    ---
    返回:
        bool : 是否创建成功凭证.
    
    """

    project_path = CONST.SYS.PATH_ROOT.joinpath(name)

    if not project_path.exists():
        project_path.mkdir(parents=True, exist_ok=True)
        return True
    else:
        return False



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Web工厂类 ]

"""

from typing import Any

from selenium.webdriver.common.by import By

from rains.kit.web.packaging.web_plant_view import WebPlantView
from rains.kit.web.packaging.web_plant_element import WebPlantElement
from rains.kit.web.packaging.web_element_structure import WebElementStructure


BY: By = By
""" [ 元素定位策略 ] """


# ----------------------------------
class WebPlant(object):
    """
    [ Web工厂类 ]

    * 提供了对浏览器视图访问与控件元素的构建封装.

    """

    _web_driver: Any
    """ [ Web驱动程序 ] """

    # ------------------------------
    def __init__(self, web_driver: Any):
        """
        [ Web工厂类 ]

        ---
        参数:
            web_driver { WebDriver } : Web 驱动程序.

        """

        self._web_driver = web_driver

    # ------------------------------
    @property
    def driver(self):
        return self._web_driver

    # ------------------------------
    @property
    def view(self) -> WebPlantView:
        """
        [ Web视图 ]

        * 封装了浏览器本身的功能, 如获取浏览器信息、弹窗控制、标签页控制等.

        ---
        返回:
            WebPlantView : Web 视图对象.

        """

        return WebPlantView(self._web_driver)

    # ------------------------------
    def _map_view(self):
        return self.view

    # ------------------------------
    def element(self, 
                page: str, 
                name: str, 
                by_key: str, 
                by_value: str, 
                anchor_by_key: str = None, 
                anchor_by_value: str = None, 
                anchor_location_id: int = None) -> WebPlantElement:
        """
        [ Web元素控件 ]

        * 基于元素特征构建指向该元素的元素控件对象.
        * 封装了对页面元素的操作功能, 如获取元素信息、模拟鼠标、输入框、下拉框等操作.
        
        ---
        参数:
            page { str } : 页面名称.
            name { str } : 元素名称.
            by_key { str } : 元素定位策略.
            by_value { str } : 元素定位策略对应的值.
            anchor_by_key { str, optional } : 锚点元素定位策略.
            anchor_by_value { str, optional } : 锚点元素定位策略对应的值.
            anchor_location_id { str, optional } : 锚点元素定位ID.

        ---
        返回:
            WebPlantElement : Web 元素控件对象.

        """

        element_structure = WebElementStructure(
            page, 
            name, 
            by_key, 
            by_value, 
            anchor_by_key, 
            anchor_by_value, 
            anchor_location_id
        )

        return WebPlantElement(self._web_driver, element_structure)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 执行服务 ]

"""

import gc
import time
import psutil
from typing import Any

from flask import jsonify
from flask import Blueprint
from tornado.ioloop import IOLoop
from tornado.wsgi import WSGIContainer
from tornado.httpserver import HTTPServer

from rains.baseic.log import LOG
from rains.baseic.config import CONFIG
from rains.server.api import RainsApp
from rains.server.api import URL_PREFIX
from rains.server.api import ServerRequestHandler
from rains.core.common.pool_runtime import PoolRuntime
from rains.core.common.driver_agency import DriverAgency

# ------------------------------
# 临时测试依赖
from rains.kit.web import WebTask
# ------------------------------


_POOL_RUNTIME: PoolRuntime = PoolRuntime()
""" [ POOL运行时 ] """

BLUEPRINT_RAINS = Blueprint('rains', __name__)
""" [ 任务接口蓝图 ] """


# 创建驱动代理
for _i in range(CONFIG.driver_max_count):
    _POOL_RUNTIME.driver_queue.put(DriverAgency())
    _POOL_RUNTIME.sign_driver()


# ------------------------------
# 临时测试依赖
@BLUEPRINT_RAINS.route(f'{ URL_PREFIX }/rains/test', methods=['GET'])
def tasks():
    
    _POOL_RUNTIME.task_queue.put(TaskTest)

    return jsonify({
        'code': 0, 
        'result': {
            'data': '[测试] 任务已接收', 
        }, 
        'message': 'ok', 
        'type': 'success'
    })
# ------------------------------


# ------------------------------
# 临时测试依赖
@BLUEPRINT_RAINS.route(f'{ URL_PREFIX }/rains/run', methods=['POST'])
def get_user_info():
    try:
        paras: dict = ServerRequestHandler.analysis_request_parameter(keys=['name'])

        if paras['name'] == 'BAIDU':
            _POOL_RUNTIME.task_queue.put(BdSearch)

        if paras['name'] == 'RAINS':
            _POOL_RUNTIME.task_queue.put(Rains)

        return jsonify({
            'code': 0, 
            'result': {
                'data': f'[{ paras["name"] }] 任务已完成'
            }, 
            'message': 'ok', 
            'type': 'success'
        })

    except BaseException as error:
        return ServerRequestHandler.unsuccessful(f'{ error }')
# ------------------------------


# ------------------------------
class PerformServer(object):
    """
    [ 执行服务 ]

    * 开启时, 将在本地挂起一个执行池进程与 WebAPI 服务.

    """

    # ------------------------------
    def __init__(self):
        """
        [ 执行服务 ]

        """
        ...

    # ------------------------------
    def running(self, port: int = 3700):
        """
        [ 运行 ]

        ---
        参数:
            port { int } : 服务端口号, 默认为 3700.

        """

        _POOL_RUNTIME.pool.apply_async(func=self.monitor)
        
        app = RainsApp()
        app.load_blueprint(BLUEPRINT_RAINS)
        
        http_server = HTTPServer(WSGIContainer(app.living))
        http_server.listen(port)

        LOG.info(f'执行服务运行中 >> 端口::[ { port } ] ')
        IOLoop.current().start()

    # ------------------------------
    def monitor(self):
        """
        [ 监听进程 ]

        * 监听来自 POOL运行时 的队列信息.

        """

        LOG.info('监听进程已启动...')

        while True:

            task = _POOL_RUNTIME.task_queue.get()
            driver = _POOL_RUNTIME.driver_queue.get()

            # 驱动冷启动时会占用大量的 CPU 资源, 为避免因为 CPU 占用率过高导致进程启动异常
            # 每隔 2 秒检查 CPU 占用率, 只有 CPU 占用率少于 50 时才会启动进程
            while True:
                if psutil.cpu_percent(2) < 50:
                    _POOL_RUNTIME.pool.apply_async(func=self._emit, args=[task, driver])
                    break

    # ------------------------------
    def _emit(self, task: Any, driver: Any):
        """
        [ 触发工作流 ]

        ---
        参数:
            task { Any } : 任务对象.
            driver { Any } : 驱动对象.

        """

        # 驱动执行任务
        driver.start_task(task)

        # 驱动执行完任务后, 放回核心队列等待
        _POOL_RUNTIME.driver_queue.put(driver)

        # 强制 GC 回收垃圾避免内存溢出
        gc.collect()



# ------------------------------
# 临时测试依赖
class TaskTest(WebTask):
    """
    测试任务
    """

    def set_task_init(self):

        print('任务 :: 执行任务起点函数')
        
        self.user_input = self._plant.element(
            page='超管后台', 
            name='邮箱输入框', 
            by_key='xpath', 
            by_value='/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
        )

        self.password_input = self._plant.element(
            page='超管后台', 
            name='密码输入框', 
            by_key='xpath', 
            by_value='/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
        )

        self.login_button = self._plant.element(
            page='超管后台', 
            name='Login按钮', 
            by_key='xpath', 
            by_value='/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
        )

        self.create_button = self._plant.element(
            page='超管后台', 
            name='Login按钮', 
            by_key='xpath', 
            by_value='/html/body/div/div/div/div/div/div[1]/a'
        )

    def set_task_quit(self):
        print('任务 :: 执行任务终点函数')

    def set_case_init(self):
        print('任务 :: 执行用例起点函数')
        self._plant.view.page.goto('http://manage.dev-tea.cblink.net/')

    def set_case_quit(self):
        print('任务 :: 执行用例终点函数')

    def case_1(self):
        """
        登录成功
        """
        self.user_input.input.send('admin@baocai.us')
        self.password_input.input.send('123456')
        self.login_button.mouse.tap()
        return self.create_button.get.exist() is True
# ------------------------------


# ------------------------------
# 临时测试依赖
class BdSearch(WebTask):
    """
    示例: 百度搜索 CBLINK
    """

    def set_task_init(self):
        self.search_input = self._plant.element(
            page='百度', 
            name='搜索输入框', 
            by_key='xpath', 
            by_value='//*[@id="kw"]'
        )

        self.search_ok = self._plant.element(
            page='百度', 
            name='搜索确认按钮', 
            by_key='xpath', 
            by_value='//*[@id="su"]'
        )

    def set_case_init(self):
        self._plant.view.page.goto('http://www.baidu.com/')

    def case_01(self):
        self.search_input.input.send('CBLINK')
        self.search_ok.mouse.tap()
        time.sleep(2)
        return True
# ------------------------------


# ------------------------------
# 临时测试依赖
class Rains(WebTask):
    """
    示例: 登录 Rains-Admin 执行脚本
    """

    def set_task_init(self):
        self.user_input = self._plant.element(
            page='Rains', 
            name='账号输入框', 
            by_key='xpath', 
            by_value='//*[@id="account"]'
        )

        self.pasword_input = self._plant.element(
            page='Rains', 
            name='密码输入框', 
            by_key='xpath', 
            by_value='//*[@id="password"]'
        )

        self.login_button = self._plant.element(
            page='Rains', 
            name='登录按钮', 
            by_key='xpath', 
            by_value='//*[@id="app"]/div/div[2]/div/div[2]/div/form/div[4]/div/div/div/button/span'
        )

        self.to_test_home_button = self._plant.element(
            page='Rains', 
            name='跳转联调', 
            by_key='xpath', 
            by_value='//*[@id="app"]/section/section/section/div[3]/div/div[2]/div/div[2]/div[1]/div[2]/div[1]/span'
        )

        self.click_running_button = self._plant.element(
            page='Rains', 
            name='点击执行脚本', 
            by_key='xpath', 
            by_value='//*[@id="app"]/section/section/section/div[3]/div/div[2]/button[1]/span'
        )

    def set_case_init(self):
        self._plant.view.set.browser_max()
        self._plant.view.page.goto('http://127.0.0.1:9001/')

    def case_01(self):
        self.user_input.input.send('admin')
        self.pasword_input.input.send('admin')
        self.login_button.mouse.tap()
        self.to_test_home_button.mouse.tap()
        self.click_running_button.mouse.tap()
        time.sleep(2)
        return True
# ------------------------------



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 运行任务 ]

"""

import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))

import time
import shutil
import pathlib
import importlib
from pathlib import Path

from rains.baseic.const import CONST


def function_run_task():
    """  
    [ 执行任务 ]

    """

    task_list = []

    if len(sys.argv) >= 3:

        if not CONST.SYS.PATH_RAINS_INI_FILE.is_file():
            print(_MESSAGE_ERROR)
            exit(0)

        # 循环参数
        for task_arg in sys.argv[2:]:

            # importlib 解析路径
            imp_analysis_str = _TASK_CACHE_NAMING

            # 拼接任务文件缓存目录路径
            # 因为 importlib 导入 py文件 的特殊方式, 需要在当前空间中创建任务文件缓存目录.
            imp_analysis_str = _TASK_CACHE_NAMING
            task_cache_path: Path = Path(sys.argv[0]).parent.joinpath(_TASK_CACHE_NAMING)
            if not task_cache_path.exists():
                task_cache_path.mkdir(0o777)

            if task_arg in ['.', '/', './']:
                task_arg_path = CONST.SYS.PATH_ROOT
            else:
                # 拼接参数路径
                task_arg_path = CONST.SYS.PATH_ROOT.joinpath(task_arg)  

            # 如果参数路径指向目录
            if task_arg_path.is_dir():
                dir_parent_name = pathlib.WindowsPath(str(task_arg_path)).parts[-1]
                save_dir = task_cache_path.joinpath(dir_parent_name)
                save_dir.mkdir(parents=True, exist_ok=True)
                imp_analysis_str += f'.{ dir_parent_name }'
                py_list = task_arg_path.glob(f'{ _LEGITIMATE_TASK_FILE_PREFIX }*.py')
                for py_file in py_list:
                    cache_task = save_dir.joinpath(py_file.name)
                    cache_task.write_bytes(py_file.read_bytes())
                    task_list += analysis_py_file_tasks(f'{ imp_analysis_str }.{ py_file.stem }')
    
            # 如果参数路径指向文件
            else:
                if task_arg_path.suffix != '.py':
                    task_arg_path = Path(f'{ str(task_arg_path) }.py')

                if task_arg_path.is_file():
                    dir_parent_name = pathlib.WindowsPath(str(task_arg_path)).parts[-2]
                    save_dir = task_cache_path.joinpath(dir_parent_name)
                    save_dir.mkdir(parents=True, exist_ok=True)
                    imp_analysis_str += f'.{ dir_parent_name }'
                    cache_task = save_dir.joinpath(task_arg_path.name)
                    cache_task.write_bytes(task_arg_path.read_bytes())
                    task_list += analysis_py_file_tasks(f'{ imp_analysis_str }.{ task_arg_path.stem }')

        if len(task_list) < 1:
            print('暂无任务可供执行!')
            shutil.rmtree(task_cache_path)
            exit(0)

        # 多进程执行任务
        from rains.core.perform_pool import PerFormPool
        perform_pool = PerFormPool()
        
        number = 1
        print('\n--------------------------------------\n')
        print('解析任务清单 : \n')
        for task in task_list:
            print(f' { number } : { task.__name__ }')
            perform_pool.put_task(task)
            number += 1
        print('\n--------------------------------------\n')
        perform_pool.running()

        # 销毁任务缓存
        shutil.rmtree(task_cache_path)


def analysis_py_file_tasks(imp_analysis_str: str) -> list:
    """
    [ 从 importlib 路径中解析任务类 ]

    ---
    参数:
        imp_analysis_str { str } : importlib 路径.

    ---
    返回:
        list : 任务类列表.

    """

    time.sleep(0.5)
    tasks = []
    module = importlib.import_module(imp_analysis_str)
    for n in module.__dict__.keys():
        if _LEGITIMATE_TASK_CLASS_PREFIX in n[0:4]:
            tasks.append(module.__dict__[n])
    return tasks


# 任务缓存目录名称
_TASK_CACHE_NAMING = 'task_cache'

# 合法的任务文件前缀
_LEGITIMATE_TASK_FILE_PREFIX = 'task'

# 合法的任务类前缀
_LEGITIMATE_TASK_CLASS_PREFIX = 'Task'

# 任务执行异常文案
_MESSAGE_ERROR = \
"""
--------------------------------------

请在工程根目录执行 -run 命令

初始化工程将创建或补全项目执行依赖文件, 并在工程根目录中创建 <rains.ini> 文件。

<rains.ini> 文件 仅作为工程根目录标识之用, -run|-server 命令需要在工程根目录中才允许执行。

--------------------------------------
"""



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 字段关键字 ]

"""


# ----------------------------------
class KeywordArg(object):
    """ [ 字段关键字 ] """

    TID: str = 'tid'
    """ [ 任务编号 ] """

    CID: str = 'cid'
    """ [ 用例编号 ] """

    NAME: str = 'name'
    """ [ 名称 ] """

    TYPE: str = 'type'
    """ [ 类型 ] """

    REMARK: str = 'remark'
    """ [ 备注信息 ] """

    CREATED_DATE: str = 'created_date'
    """ [ 创建日期 ] """

    STATE: str = 'state'
    """ [ 状态 ] """

    START_TIME: str = 'start_time'
    """ [ 开始时间 ] """

    END_TIME: str = 'end_time'
    """ [ 结束时间 ] """

    SPEND_TIME_S: str = 'spend_time_s'
    """ [ 消耗时间(秒) ] """

    CASE_ALL: str = 'case_all'
    """ [ 所有用例计数 ] """

    CASE_PASS: str = 'case_pass'
    """ [ 成功的用例计数 ] """

    CASE_FAIL: str = 'case_fail'
    """ [ 失败的用例计数 ] """

    RUN_STEP: str = 'run_step'
    """ [ 运行步骤 ] """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Web 驱动程序 ]

"""

import random

from selenium import webdriver
from selenium.webdriver.chrome.webdriver import RemoteWebDriver

from rains.baseic.const import CONST
from rains.baseic.interface.i_driver import IDriver
from rains.core.common.task_executor import TaskExecutor
from rains.kit.web.web_plant import WebPlant
from rains.kit.web.packaging.web_find_handle import WebFindHandle


WEB_DRIVER_SIGN_ID_LIST = []
""" [ 驱动程序编号池 ] """


# ----------------------------------
class WebDriver(IDriver):
    """
    [ Web 驱动程序 ]

    """
    
    _driver_id: int
    """ [ 驱动程序ID ] """

    _plant: WebPlant = None
    """ [ 构建工厂对象 ] """
    
    _state: bool = False
    """ [ 运行状态 ] """

    _task_executor: TaskExecutor
    """ [ 任务执行器 ] """
    
    _base_driver: RemoteWebDriver
    """ [ 原生驱动对象 ] """

    _find_handle: WebFindHandle
    """ [ 元素定位器 ] """

    _project_type: str = CONST.TYPE.PROJECT_TYPE_WEB
    """ [ 项目类型 ] """

    # ------------------------------
    def __init__(self):
        """
        [ Web 驱动程序 ]

        """

        # 生成随机ID并创建驱动程序
        while True:
            self._driver_id = random.randint(100, 999)

            if self._driver_id not in WEB_DRIVER_SIGN_ID_LIST:
                WEB_DRIVER_SIGN_ID_LIST.append(self._driver_id)
                break
            else:
                continue


        self._plant = WebPlant(self)
        self._task_executor = TaskExecutor(self)

    # ------------------------------
    def __enter__(self):
        """ 
        [ with enter ]

        * with 上下文实现.

        ---
        示例:
            >>> with WebDriver() as driver:
            >>>     driver.running()
            :: 执行完 with 内的语句后, 将执行 __exit__.
        
        """

        return self

    # ------------------------------
    def __exit__(self, exc_type, exc_val, exc_tb):
        """ 
        [ with exit ] 
        
        """

        self.end()

    # ------------------------------
    @property
    def plant(self):
        """
        [ 构建工厂 ]

        """

        return self._plant

    # ------------------------------
    @property
    def driver_id(self):
        """
        [ 驱动ID ]

        """

        return self._driver_id
    
    # ------------------------------
    @property
    def base_driver(self):
        """
        [ 原生驱动对象 ]

        """

        return self._base_driver
    
    # ------------------------------
    @property
    def find_handle(self):
        """
        [ 元素定位器 ]

        """

        return self._find_handle

    # ------------------------------
    @property
    def project_type(self):
        """
        [ 项目类型 ]

        """

        return self._project_type

    # ------------------------------
    def get_state(self) -> bool:
        """
        [ 获取引擎运行状态 ]

        * 获取引擎运行状态, 启动中返回 True, 否则返回 False. 

        Returns:
            bool: [ 引擎运行状态 ]

        """

        return self._state
    
    # ------------------------------
    def start_task(self, task):
        """
        [ 开始任务 ]

        * 接收一个 task, 并执行该 task.

        ---
        参数:
            task { Any } : 任务对象, 可以是实现 ITask 接口的任务类, 或者是 JSON 串译文.

        """

        self._task_executor.running(task)
        
        
    # ------------------------------
    def running(self, browser: str = 'CHROME'):
        """
        [ 运行引擎 ]

        * 驱动与定位器的实例化.

        """

        if not self._state:

            # 谷歌浏览器
            if browser == 'CHROME':

                # 创建浏览器驱动实例
                opt = webdriver.ChromeOptions()

                # 以非 W3C 模式运行
                opt.add_experimental_option('w3c', False)

                # 不显示自动化劫持 && 不打印日志
                opt.add_experimental_option('excludeSwitches', ['enable-automation', 'enable-logging'])

                self._base_driver = webdriver.Chrome(options=opt)
                self._find_handle = WebFindHandle(self._base_driver)
                self._state = True

    # ------------------------------
    def end(self):
        """
        [ 结束引擎 ]

        * 退出并销毁当前持有的驱动与定位器. 

        """

        if self._state:

            self._base_driver.quit()
            self._state = False

            del self._base_driver
            del self._find_handle

    # ------------------------------
    def reset(self):
        """
        [ 重置引擎 ]

        * 将引擎重置至初始阶段.
        * 与 end() 相比, 该函数只是重置引擎至初始状态, 而不是注销.

        """

        if self._state:

            # 打开一个新空白标签页
            self._base_driver.execute_script(f'window.open("")')

            # 关闭所有其他标签页面
            new_page_handle = self._base_driver.window_handles[-1]

            for page_handle in self._base_driver.window_handles:
                if page_handle != new_page_handle:
                    self._base_driver.switch_to.window(page_handle)
                    self._base_driver.close()

            # 句柄回到新空白标签页
            self._base_driver.switch_to.window(new_page_handle)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 初始化工程 ]

"""

import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))

from rains.baseic.environment.make import create_runtime_environment
from rains.baseic.environment.make import init_engineering


_MESSAGE = \
"""
--------------------------------------

工程初始化成功。

初始化工程将创建或补全项目执行依赖文件, 并在工程根目录中创建 <rains.ini> 文件。

<rains.ini> 文件 仅作为工程根目录标识之用, -run|-server 命令需要在工程根目录中才允许执行。

--------------------------------------
"""


def function_init_engineering():
    """
    [ 初始化工程 ]

    """

    create_runtime_environment()
    init_engineering()
    print(_MESSAGE)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Kit.Web.Packaging.WebPlantView ]

"""

from typing import Any

from selenium.common.exceptions import NoAlertPresentException
from selenium.common.exceptions import InvalidArgumentException

from rains.baseic.log import LOG
from rains.baseic.config import CONFIG
from rains.baseic.tools import Regular
from rains.core.common.action_recorder import ACTION_RECORDER
        

# ----------------------------------
class WebPlantView(object):
    """
    [ Web视图 ]

    * 封装了浏览器本身的功能, 如获取浏览器信息、弹窗控制、标签页控制等.

    """

    _web_driver: Any
    """ [ 引擎对象 ] """

    # ------------------------------
    def __init__(self, web_driver: Any):
        """
        [ Web视图 ]

        ---
        参数:
            web_driver { WebEngine } : Web引擎对象.

        """

        self._web_driver = web_driver

    # ------------------------------
    @property
    def get(self):
        """
        [ Web视图'获取'相关函数 ]

        """

        return _WebPlantViewMapGet(self._web_driver)

    # ------------------------------
    def _map_get(self):
        return self.get

    # ------------------------------
    @property
    def set(self):
        """
        [ Web视图'设置'相关函数 ]

        """

        return _WebPlantViewMapSet(self._web_driver)

    # ------------------------------
    def _map_set(self):
        return self.set

    # ------------------------------
    @property
    def page(self):
        """
        [ Web视图'页面'相关函数 ]

        """

        return _WebPlantViewMapPage(self._web_driver)

    # ------------------------------
    def _map_page(self):
        return self.page

    # ------------------------------
    @property
    def alert(self):
        """
        [ Web视图'弹窗'相关函数 ]

        """

        return _WebPlantViewMapAlert(self._web_driver)

    # ------------------------------
    def _map_alert(self):
        return self.alert


# ----------------------------------
class _WebPlantViewMapBase(object):
    """
    [ Web视图映射基础类 ]

    """

    _handler: Any
    _driver_id: int

    # ------------------------------
    def __init__(self, web_driver: Any):
        """
        [ Web视图映射基础类 ]

        ---
        参数:
            web_driver { WebDriver } : Web驱动对象.

        """

        self._handler = web_driver.base_driver
        self._driver_id = web_driver.driver_id


# ----------------------------------
class _WebPlantViewMapGet(_WebPlantViewMapBase):
    """
    [ Web视图'获取'相关函数 ]

    """

    # ------------------------------
    def url(self) -> str:
        """
        [ 获取当前标签页的 URL ]

        ---
        返回:
            str : 当前标签页的 URL 地址.

        """

        try:
            message = Regular.get_function_doc_name(self.url.__doc__)
            current_url = self._handler.current_url
            execution_step_record(self._driver_id, message, current_url)

            return current_url

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def title(self) -> str:
        """
        [ 获取当前标签页的 Title ]

        ---
        返回:
            str : 当前标签页的 Title 名称.

        """

        try:
            message = Regular.get_function_doc_name(self.title.__doc__)
            current_title = self._handler.title
            execution_step_record(self._driver_id, message, current_title)

            return current_title

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def window_list(self) -> list:
        """
        [ 获取标签页句柄列表 ]

        ---
        返回:
            list : 标签页句柄列表.

        """

        try:
            message = Regular.get_function_doc_name(self.window_list.__doc__)
            window_handles = self._handler.window_handles
            execution_step_record(self._driver_id, message, f'len:{ len(window_handles) }')

            return window_handles

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def window_current(self) -> Any:
        """
        [ 获取当前标签页句柄 ]

        ---
        返回:
            Any : 当前标签页句柄对象.

        """

        try:
            message = Regular.get_function_doc_name(self.window_current.__doc__)
            current_window_handle = self._handler.current_window_handle
            execution_step_record(self._driver_id, message, current_window_handle)

            return current_window_handle

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def browser_size(self) -> dict:
        """
        [ 获取浏览器尺寸 ]

        ---
        返回:
            dict : 标识浏览器尺寸信息的字典.

        """

        try:
            message = Regular.get_function_doc_name(self.browser_size.__doc__)
            browser_size = self._handler.get_window_size()
            execution_step_record(self._driver_id, message, browser_size)

            return browser_size

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def browser_position(self) -> dict:
        """
        [ 获取浏览器位置坐标 ]
        
        ---
        返回:
            dict : 标识浏览器位置坐标信息的字典.

        """

        try:
            message = Regular.get_function_doc_name(self.browser_position.__doc__)
            browser_position = self._handler.get_window_position()
            execution_step_record(self._driver_id, message, browser_position)

            return browser_position

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def cookies(self) -> list:
        """
        [ 获取 Cookies ]
        
        ---
        返回:
            list : Cookies 列表.

        """

        try:
            message = Regular.get_function_doc_name(self.cookies.__doc__)
            cookies = self._handler.get_cookies()
            execution_step_record(self._driver_id, message, len(cookies))

            return cookies

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)


# ----------------------------------
class _WebPlantViewMapSet(_WebPlantViewMapBase):
    """
    [ WEB视图'设置'相关函数 ]

    """

    # ------------------------------
    def browser_size(self, width: int, height: int):
        """
        [ 设置浏览器尺寸 ]

        ---
        参数:
            width { int } : 宽度.
            height { int } : 高度.

        """

        try:
            message = Regular.get_function_doc_name(self.browser_size.__doc__)
            self._handler.set_window_size(width, height)
            execution_step_record(self._driver_id, message, {'width': width, 'height': height})

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def browser_position(self, x: int, y: int):
        """
        [ 设置浏览器位置坐标 ]

        ---
        参数:
            x { int } : x坐标.
            y { int } : y坐标.

        """

        try:
            message = Regular.get_function_doc_name(self.browser_position.__doc__)
            self._handler.set_window_position(x, y)
            execution_step_record(self._driver_id, message, {'x': x, 'y': y})

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def browser_max(self):
        """
        [ 设置浏览器窗口最大化 ]

        """

        try:
            message = Regular.get_function_doc_name(self.browser_max.__doc__)
            self._handler.maximize_window()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def browser_min(self):
        """
        [ 设置浏览器窗口最小化 ]

        """

        try:
            message = Regular.get_function_doc_name(self.browser_min.__doc__)
            self._handler.minimize_window()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def del_cookies(self):
        """
        [ 删除所有 Cookies ]

        """

        try:
            message = Regular.get_function_doc_name(self.del_cookies.__doc__)
            self._handler.delete_all_cookies()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def add_cookies(self, cookie_list: list):
        """
        [ 添加 Cookies ]

        ---
        参数:
            cookie_list { list } : Cookies 列表.

        """

        try:
            message = Regular.get_function_doc_name(self.add_cookies.__doc__)

            for cookie in cookie_list:
                self._handler.add_cookie(cookie)
            execution_step_record(self._driver_id, message, len(cookie_list))

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)


# ----------------------------------
class _WebPlantViewMapPage(_WebPlantViewMapBase):
    """
    [ Web视图'页面'相关函数 ]

    """

    # ------------------------------
    def goto(self, url: str):
        """
        [ 跳转URL ]

        * 当前标签页跳转URL.

        ---
        参数:
            url { str } : URL.

        """

        try:
            message = Regular.get_function_doc_name(self.goto.__doc__)
            self._handler.get(url)
            execution_step_record(self._driver_id, message, url)

        except InvalidArgumentException:
            raise WebPlantViewFunctionRunError(self._driver_id, message, 'URL路径格式错误')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def refresh(self):
        """
        [ 刷新页面 ]

        * 当前标签页刷新页面.

        """

        try:
            message = Regular.get_function_doc_name(self.refresh.__doc__)
            self._handler.refresh()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def back(self):
        """
        [ 回退页面 ]

        * 当前标签页回退页面.

        """

        try:
            message = Regular.get_function_doc_name(self.back.__doc__)
            self._handler.back()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def forward(self):
        """
        [ 前滚页面 ]

        * 当前标签页前滚页面.

        """

        try:
            message = Regular.get_function_doc_name(self.forward.__doc__)
            self._handler.forward()
            execution_step_record(self._driver_id, message)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def save_png(self, save_path: str):
        """
        [ 保存快照 ]

        * 将当前浏览器截图保存至 save_path 中.

        ---
        参数:
            save_path { str } : 快照的完整保存路径.

        """

        try:
            message = Regular.get_function_doc_name(self.save_png.__doc__)
            self._handler.get_screenshot_as_file(str(save_path))
            execution_step_record(self._driver_id, message, save_path)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def window_open(self, url: str = ''):
        """
        [ 打开标签页 ]

        * 在当前浏览器中, 打开一个新的标签页.

        ---
        参数:
            url { str } : 新标签页的URL, 默认为空.

        """

        try:
            message = Regular.get_function_doc_name(self.window_open.__doc__)
            self._handler.execute_script(f'window.open("{url}")')
            self._handler.switch_to.window(self._handler.window_handles[-1])
            execution_step_record(self._driver_id, message, url)

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def window_goto(self, window_id: int):
        """
        [ 切换标签页 ]

        * 在当前浏览器中, 切换至编号为 [window_id] 的标签页.

        ---
        参数:
            window_id { int } : 标签页编号.

        """

        try:
            message = Regular.get_function_doc_name(self.window_goto.__doc__)
            self._handler.switch_to.window(self._handler.window_handles[window_id])
            execution_step_record(self._driver_id, message, self._handler.current_url)

        except IndexError:
            raise WebPlantViewFunctionRunError(self._driver_id, message, '标签页编号不存在')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def window_close(self, window_id: int):
        """
        [ 关闭标签页 ]

        * 在当前浏览器中, 关闭编号为 [window_id] 的已存在标签页.

        ---
        参数:
            window_id { int } : 标签页编号.

        """

        try:
            message = Regular.get_function_doc_name(self.window_close.__doc__)
            self._handler.switch_to.window(self._handler.window_handles[window_id])
            current_url = self._handler.current_url
            self._handler.close()
            self._handler.switch_to.window(self._handler.window_handles[0])
            execution_step_record(self._driver_id, message, current_url)

        except IndexError:
            raise WebPlantViewFunctionRunError(self._driver_id, message, '标签页编号不存在')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)


# ----------------------------------
class _WebPlantViewMapAlert(_WebPlantViewMapBase):
    """
    [ Web视图'弹窗'相关函数 ]

    """

    # ------------------------------
    def exist(self) -> bool:
        """
        [ 检查弹窗是否存在 ]

        ---
        返回:
            bool : True or False.

        """

        try:
            message = Regular.get_function_doc_name(self.exist.__doc__)
            is_exist = True

            try:
                self._handler.switch_to.alert
            except NoAlertPresentException:
                is_exist = False

            execution_step_record(self._driver_id, message, str(is_exist))

            return is_exist

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def text(self) -> str:
        """
        [ 获取弹窗文本 ]

        ---
        返回:
            str : 弹窗文本字符串.

        """

        try:
            message = Regular.get_function_doc_name(self.text.__doc__)
            alert_text = self._handler.switch_to.alert.text
            execution_step_record(self._driver_id, message, alert_text)

            return alert_text

        except NoAlertPresentException:
            raise WebPlantViewFunctionRunError(self._driver_id, message, '当前没有弹窗')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def tap_accept(self):
        """
        [ 点击弹窗确定按钮 ]

        """

        try:
            message = Regular.get_function_doc_name(self.tap_accept.__doc__)
            self._handler.switch_to.alert.accept()
            execution_step_record(self._driver_id, message)

        except NoAlertPresentException:
            raise WebPlantViewFunctionRunError(self._driver_id, message, '当前没有弹窗')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # ------------------------------
    def tap_dismiss(self):
        """
        [ 点击弹窗取消按钮 ]

        """
        
        try:
            message = Regular.get_function_doc_name(self.tap_dismiss.__doc__)
            self._handler.switch_to.alert.dismiss()
            execution_step_record(self._driver_id, message)

        except NoAlertPresentException:
            raise WebPlantViewFunctionRunError(self._driver_id, message, '当前没有弹窗')

        except BaseException as error:
            raise WebPlantViewFunctionRunError(self._driver_id, message, error)

    # # ------------------------------
    # def send_key(self, key: str):
    #     """
    #     [ 弹窗输入框键入文本 ]

    #     ---
    #     参数:
    #         key { str } : 文本.

    #     """
        
    #     try:
    #         message = Regular.get_function_doc_name(self.send_key.__doc__)
    #         self._handler.switch_to.alert.send_keys(key)
    #         execution_step_record(self._driver_id, message, key)

    #     except NoAlertPresentException:
    #         raise WebPlantViewFunctionRunError(self._driver_id, message, '当前没有弹窗')

    #     except BaseException as error:
    #         raise WebPlantViewFunctionRunError(self._driver_id, message, error)


# ----------------------------------
def execution_step_record(driver_id: int, message: str, join_message: str = None, level: str = 'info'):
    """
    [ 执行步骤记录 ]

    * 执行时, 将同步将信息传递给日志服务与用例记录器.

    ---
    参数:
        driver_id { int } : 核心ID.
        message { str } : 记录信息.
        join_message { str } : 拼接信息.

    """

    if join_message:
        message = ''.join([message, ' >> ', f'[{ str(join_message) }]'])

    ACTION_RECORDER.write(driver_id, message)

    if CONFIG.debug:
        LOG.map(level, f'Driver[{ driver_id }] >> { message }')


# ----------------------------------
class WebPlantViewFunctionRunError(Exception):
    """ [ Web视图函数执行异常 ] """
    
    def __init__(self, driver_id: int, message: str, error: Any):
        self.message = f'error :: { message } :: { error } !'
        self.err_name = Regular.get_function_doc_name(self.__doc__)
        execution_step_record(driver_id, message, level='error')

    def __str__(self) -> str:
        return ''.join([self.err_name, ' :: ', self.message])



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 装饰器实现 ]

"""


# ----------------------------------
def singleton_pattern(cls):
    """
    [ 单例模式装饰器 ]

    * 用于装饰类, 被装饰的类在全局引用时只会创建一个实例.

    ---
    示例:
        >>> @singleton_pattern
        >>> class T(object): ...
        >>> t1 = T()
        >>> t2 = T()
        >>> id(t1) == id(t2) :: True

    """

    instances = {}

    def _singleton_pattern(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return _singleton_pattern



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 驱动器接口 ]

"""

from typing import Any

from abc import ABCMeta
from abc import abstractmethod


# ------- I DRIVER -------
class IDriver(metaclass=ABCMeta):
    """
    [ 套件驱动程序接口 ]
    
    """

    # ------------------------------
    @abstractmethod
    def plant(self):
        """
        [ 构建工厂 ]

        """
        ...

    # ------------------------------
    @abstractmethod
    def driver_id(self):
        """
        [ 驱动ID ]

        """
        ...

    # ------------------------------
    @abstractmethod
    def get_state(self) -> bool:
        """
        [ 获取驱动程序运行状态 ]

        * 获取当前驱动程序运行状态, 启动中返回 True, 否则返回 False.

        ---
        返回:
            bool : 当前引擎状态.

        """
        ...

    # ------------------------------
    @abstractmethod
    def start_task(self, task: Any):
        """
        [ 开始任务 ]

        * 接收一个 task, 并执行该 task.

        ---
        参数:
            task { Any } : 任务对象, 可以是实现 ITask 接口的任务类, 或者是 JSON 译文.

        """
        ...

    # ------------------------------
    @abstractmethod
    def running(self):
        """
        [ 运行 ]
        
        * 运行驱动程序.

        """
        ...

    # ------------------------------
    @abstractmethod
    def end(self):
        """
        [ 结束 ]

        * 结束驱动程序.

        """
        ...

    # ------------------------------
    @abstractmethod
    def reset(self):
        """
        [ 重置 ]

        * 重置驱动程序.
        * 与结束驱动程序相比, 该接口只需要将驱动程序重置为初始状态即可.

        """
        ...



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 任务外交代理 ]

"""

import time
import json

from rains.server.db import DB
from rains.server.db import SQL
from rains.baseic.const import CONST
from rains.baseic.keyword import KEYWORD
from rains.core.common.action_recorder import ACTION_RECORDER


# ----------------------------------
class TaskDiplomatist(object):
    """
    [ 任务外交代理 ]

    * 该类将代理任务执行过程中的数据库写入事务以及日志输出.

    """

    _driver_id: int
    """ [ 驱动ID ] """

    _task_id: int
    """ [ 任务ID ] """

    _task_type: str
    """ [ 任务类型 ] """

    _task_created_date: str
    """ [ 任务创建日期 ] """

    _task_start_time: str
    """ [ 任务开始时间 ] """

    _task_start_timestamp: int
    """ [ 任务开始时间戳 ] """

    _case_all_count: int = 0
    """ [ 所有用例计数 ] """

    _case_pass_count: int = 0
    """ [ 通过用例计数 ] """

    _case_fail_count: int = 0
    """ [ 失败用例计数 ] """

    # ------------------------------
    def __init__(self, driver_id: int, task_type: str):
        """
        [ 任务外交代理 ]
        
        ---
        参数:
            driver_id { int } : 核心ID.
            task_type { str } : 任务类型.
        
        """

        self._driver_id = driver_id
        self._task_type = task_type

    # ------------------------------
    def created_task(self, task_name: str, task_remark: str) -> int:
        """
        [ 记录任务 ]

        ---
        参数:
            task_name { str } : 任务名称.
            task_remark { str } : 任务注释.
        
        """

        # 记录任务开始时间
        self._task_created_date = time.strftime('%Y-%m-%d')
        self._task_start_time = time.strftime('%H:%M:%S')
        self._task_start_timestamp = int(time.time())

        # 录入数据库任务记录
        self._task_id = DB.write(SQL.task.add({
            KEYWORD.ARG.NAME: task_name,
            KEYWORD.ARG.TYPE: self._task_type,
            KEYWORD.ARG.REMARK: task_remark,
            KEYWORD.ARG.CREATED_DATE: self._task_created_date,
            KEYWORD.ARG.STATE: CONST.STATE.RUN
        }))

        return self._task_id

    # ------------------------------
    def created_case(self, case_name: str, case_remark: str) -> int:
        """
        [ 记录用例 ]

        ---
        参数:
            case_name { str } : 用例名称.
            case_remark { str } : 用例注释.

        """

        case_id = DB.write(SQL.case.add({
            KEYWORD.ARG.TID: self._task_id,
            KEYWORD.ARG.NAME: case_name,
            KEYWORD.ARG.REMARK: case_remark,
            KEYWORD.ARG.CREATED_DATE: self._task_created_date
        }))

        self._case_all_count += 1

        return case_id

    # ------------------------------
    def update_case(self, cid: int, state: str, start_time: str, start_timestamp: int):
        """
        [ 更新用例 ]
        
        ---
        参数:
            cid { int } : 用例ID.
            state { str } : 用例状态.
            start_time { str } : 开始时间.
            start_timestamp { str } : 开始时间戳.

        """

        # 录入数据库更新用例
        DB.write(SQL.case.update({
            KEYWORD.ARG.CID: cid, 
            KEYWORD.ARG.STATE: state, 
            KEYWORD.ARG.START_TIME: start_time, 
            KEYWORD.ARG.END_TIME: time.strftime('%H:%M:%S'), 
            KEYWORD.ARG.SPEND_TIME_S: (int(time.time()) - start_timestamp), 
            KEYWORD.ARG.RUN_STEP: json.dumps(ACTION_RECORDER.take(self._driver_id))
        }))

        if state == CONST.STATE.END:
            self._case_pass_count += 1

        if state == CONST.STATE.ERR:
            self._case_fail_count += 1

    # ------------------------------
    def end(self, state: str):
        """
        [ 结束 ]

        ---
        参数:
            state { str } : 任务状态.
        
        """

        task_end_time = time.strftime('%H:%M:%S')

        # 更新数据库任务记录
        DB.write(SQL.task.update({
            KEYWORD.ARG.TID: self._task_id, 
            KEYWORD.ARG.STATE: state, 
            KEYWORD.ARG.START_TIME: self._task_start_time, 
            KEYWORD.ARG.END_TIME: task_end_time, 
            KEYWORD.ARG.SPEND_TIME_S: (int(time.time()) - self._task_start_timestamp), 
            KEYWORD.ARG.CASE_ALL: self._case_all_count, 
            KEYWORD.ARG.CASE_PASS: self._case_pass_count, 
            KEYWORD.ARG.CASE_FAIL: self._case_fail_count
        }))

        # 保存事务
        DB.commit()



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ POOL 运行时 ]

"""

from multiprocessing.pool import ThreadPool
from multiprocessing.dummy import Pool
from multiprocessing.dummy import Lock
from multiprocessing.dummy import Queue
from multiprocessing.dummy import Manager

from rains.baseic.config import CONFIG


# ----------------------------------
class PoolRuntime(object):
    """
    [ POOL 运行时 ]

    """

    _pool: ThreadPool
    """ [ 进程池 ] """

    _state: bool
    """ [ 运行状态 ] """

    _driver_max_count: int
    """ [ 驱动队列最大值 ] """

    _task_max_count: int
    """ [ 任务队列最大值 ] """

    _sign_driver_count: int
    """ [ 注册的驱动数量 ] """
    
    _driver_queue: Queue
    """ [ 驱动队列 ] """

    _task_queue: Queue
    """ [ 任务队列 ] """

    _state_lock: Lock
    """ [ 运行状态互斥锁 ] """

    _driver_queue_lock: Lock
    """ [ 驱动队列互斥锁 ] """

    _task_queue_lock: Lock
    """ [ 任务队列互斥锁 ] """

    # ------------------------------
    def __init__(self, 
        driver_max_count: int = CONFIG.driver_max_count, 
        task_max_count: int = CONFIG.task_max_count
    ) -> None:
        """
        [ POOL 运行时 ]
        
        ---
        参数:
            driver_max_count { int, optional } : 驱动队列最大值.
            task_max_count { int, optional } : 任务队列最大值.

        """

        self._driver_max_count = driver_max_count
        self._task_max_count = task_max_count
        self.initialization_parameter()

    # ------------------------------
    def initialization_parameter(self):
        """
        [ 初始化参数 ]

        """

        manager: Manager = Manager()

        self._pool = Pool(self._driver_max_count + 1)
        self._state = False
        self._sign_driver_count = 0
        
        self._driver_queue = manager.Queue(maxsize=self._driver_max_count)
        self._task_queue = manager.Queue(maxsize=self._task_max_count)
        self._state_lock = manager.Lock()
        self._driver_queue_lock = manager.Lock()
        self._task_queue_lock = manager.Lock()

    # ------------------------------
    @property
    def pool(self) -> ThreadPool:
        """
        [ 进程池 ]

        """

        return self._pool

    # ------------------------------
    @property
    def state(self) -> bool:
        """
        [ 运行状态 ]

        """

        with self._state_lock:
            return self._state

    # ------------------------------
    @property
    def driver_max_count(self) -> int:
        """
        [ 驱动队列最大值 ]

        """

        return self._driver_max_count

    # ------------------------------
    @property
    def task_max_count(self) -> int:
        """
        [ 任务队列最大值 ]

        """

        return self._task_max_count

    # ------------------------------
    @property
    def sign_driver_count(self) -> int:
        """
        [ 注册的核心数量 ]

        """

        return self._sign_driver_count

    # ------------------------------
    @property
    def driver_queue(self) -> Queue:
        """
        [ 核心队列 ]

        """

        with self._driver_queue_lock:
            return self._driver_queue

    # ------------------------------
    @property
    def task_queue(self) -> Queue:
        """
        [ 任务队列 ]

        """

        with self._task_queue_lock:
            return self._task_queue

    # ------------------------------
    def set_state(self, state: bool):
        """
        [ 设置运行状态 ]
        
        ---
        参数:
            state { bool } : 运行状态.

        """

        with self._state_lock:
            self._state = state

    # ------------------------------
    def sign_driver(self):
        """
        [ 注册核心 ]

        """

        self._sign_driver_count += 1




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from rains.kit.api.api_task import ApiTask
from rains.kit.api.api_driver import ApiDriver


class T(ApiTask):
    
    def case_01(self):
        r = self.plant.get('http://127.0.0.1:5000/get1')
        return r.data['code'] == 0
    
    def case_02(self):
        r = self.plant.get('http://127.0.0.1:5000/get1')
        return r.data['code'] == 0


api_driver = ApiDriver()
api_driver.start_task(T)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Api.ServerRequestHandler ]

"""

import json
from typing import Union
from werkzeug.datastructures import ImmutableMultiDict

from flask import jsonify
from flask import request
from flask import Response

from rains.baseic.const import CONST
from rains.baseic.error import ServerAnalysisParameterError
from rains.baseic.error import ServerInspectionParameterError


URL_PREFIX: str = '/api'
""" [ URL 前缀 ] """


# ----------------------------------
class ServerRequestHandler(object):
    """
    [ 服务端请求处理程序 ]
    
    * 处理 WEBAPI 请求的相关事务, 如解析前端请求参数与返回响应对象.

    """

    # ------------------------------
    @staticmethod
    def analysis_request_parameter(keys: list, must_keys: Union[list, None] = None) -> dict:
        """
        [ 解析请求参数 ]

        * 解析请求中指定 Key, 如果该值为空, 则舍弃该 Key, 而不是返回 None.

        * 如果需要确保解析结果中必须携带指定 Key, 请通过 must_keys 定义.

        ---
        示例:
            >>> params = ServerParameterHandler.analysis_request_parameter(keys=['key1', 'key2'])
            :: 该示例表示解析请求中的 key1 与 key2, 如果请求中缺失 key2, 则只返回 key1.

            >>> params = ServerParameterHandler.analysis_request_parameter(keys=['key1', 'key2'], must_keys=['key1])
            :: 该示例表示获取请求中的 key1 与 key2, 并且确保 key1 必须存在, 否则将引发异常.

        ---
        参数:
            keys { list } : 待解析的 Key 列表.
            must_keys { Union[list, None], optional } : 强制携带的 Key 列表, 默认为空.

        ---
        异常:
            ServerAnalysisParameterError : 服务端解析参数异常.
            ServerInspectionParameterError : 服务端检查参数异常.

        ---
        返回:
            dict : 返回解析完成的参数字典.

        """

        request_handle = request.args
        if request.method == 'POST':
            try:
                request_handle = json.loads(request.data.decode('UTF-8'))
            except json.decoder.JSONDecodeError:
                request_handle = request.form

            # if isinstance(request.form, ImmutableMultiDict):
            #     request_handle = json.loads(request.data.decode('UTF-8'))
            # else:
            #     request_handle = request.form

        return ServerRequestHandler._common_analysis_request_keys(request_handle, keys, must_keys)

    # ------------------------------
    @staticmethod
    def analysis_request_headers(keys: list, must_keys: Union[list, None] = None) -> dict:
        """
        [ 解析请求头 ]

        * 解析请求头中指定 Key, 如果该值为空, 则舍弃该 Key, 而不是返回 None.
        * 如果需要确保解析结果中必须携带指定 Key, 请通过 must_keys 定义.

        ---
        示例:
            >>> params = ServerParameterHandler.analysis_request_headers(keys=['Authorization'])
            :: 该示例表示获取请求头中的 Authorization 字段, 如果请求中缺失 Authorization, 则返回空字典.

            >>> params = ServerParameterHandler.analysis_request_headers(keys=['Authorization'], must_keys=['Authorization])
            :: 该示例表示获取请求头中的 Authorization 字段, 并且确保 Authorization 必须存在, 否则将引发异常.

        ---
        参数:
            keys { list } : 待解析的 Key 列表.
            must_keys { Union[list, None], optional } : 强制携带的 Key 列表, 默认为空.

        ---
        异常:
            ServerAnalysisParameterError : 服务端解析参数异常.
            ServerInspectionParameterError : 服务端检查参数异常.

        ---
        返回:
            dict : 返回解析完成的参数字典.

        """

        headers_handle = request.headers
        return ServerRequestHandler._common_analysis_request_keys(headers_handle, keys, must_keys)

    # ------------------------------
    @staticmethod
    def successful(paras: dict) -> Response:
        """
        [ 请求成功 ]

        * 将数据序列化为 Json 并包装成 Flask.Response 响应体返回.

        ---
        参数:
            paras { dict } : 需要返回的数据.

        ---
        返回:
            Response : 响应体对象, 通过 application/json 进行操作.
         
        """

        return jsonify({
            'code'    : CONST.API.CODE_SUCCESS, 
            'message' : CONST.API.MESSAGE_SUCCESS, 
            'result'  : paras
        })

    # ------------------------------
    @staticmethod
    def unsuccessful(error: str) -> Response:
        """
        [ 请求失败 ]

        * 将数据序列化为 Json 并包装成 Flask.Response 响应体返回.

        ---
        参数:
            error { str } : 需要返回的错误内容.

        ---
        返回:
            Response : 响应体对象, 通过 application/json 进行操作.
         
        """

        return jsonify({
            'code'    : CONST.API.CODE_UNSUCCESS, 
            'message' : CONST.API.MESSAGE_UNSUCCESS, 
            'result'  : error
        })

    # ------------------------------
    @staticmethod
    def _common_analysis_request_keys(data: dict, keys: list, must_keys: list) -> dict:
        """
        [ 通用的解析参数 ]

        * analysis_request_parameter and analysis_request_headers common function.

        ---
        参数:
            data { dict } : 待解析的数据.
            keys { list } : 待解析的 Key 列表. 
            must_keys { list } : 强制携带的 Key 列表.
        
        ---
        异常:
            ServerAnalysisParameterError : 服务端解析参数异常.
            ServerInspectionParameterError : 服务端检查参数异常.

        """

        try:
            decryption_parameter: dict = {}

            # 如果 key 为空, 则丢弃, 否则储存.
            for key in keys:
                value = data.get(key)
                if not value: 
                    ...  
                else:
                    decryption_parameter.update({key: value})
            
            # 检查 keys 中是否包含所有 must_keys 的成员元素, 
            # 否则抛出 ServerAnalysisParameterError 异常.
            if must_keys:
                try:
                    for key in must_keys:
                        if not decryption_parameter[key]:
                            ...
                except KeyError as error:
                    raise ServerInspectionParameterError(error)

            return decryption_parameter

        except ServerInspectionParameterError:
            raise

        except BaseException as error:
            raise ServerAnalysisParameterError(error)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 执行池 ]

"""

import gc
import psutil
from typing import Any

from rains.baseic.log import LOG
from rains.baseic.decorator import singleton_pattern
from rains.core.common.pool_runtime import PoolRuntime
from rains.core.common.driver_agency import DriverAgency


# ----------------------------------
@singleton_pattern
class PerFormPool(object):
    """
    [ 执行池 ]

    * 通常用于静态执行多进程任务.

    """

    _debug: bool
    """ [ 调试模式 ] """

    _config_runtime: PoolRuntime
    """ [ POOL运行时 ] """

    # ------------------------------
    def __init__(self, 
        debug: bool = False,
        driver_max_count: int = 3,
        task_max_count: int = 50
    ) -> None:
        """
        [ 执行池 ]

        ---
        参数:
            debug { bool, optional } : 调试模式.
            driver_max_count { int, optional } : 驱动最大数量.
            task_max_count { int, optional } : 任务最大数量.

        """

        self._debug = debug
        self._config_runtime = PoolRuntime(driver_max_count, task_max_count)

        # 创建驱动代理
        for _i in range(driver_max_count):
            self._config_runtime.driver_queue.put(DriverAgency())
            self._config_runtime.sign_driver()

    # ------------------------------
    def put_task(self, task: Any):
        """
        [ 推送到任务队列 ]
        
        ---
        参数:
            task { Any } : 任务对象.

        """

        count = self._config_runtime.task_queue.qsize()
        max_count = self._config_runtime.task_queue.maxsize
        if max_count > count:
            self._config_runtime.task_queue.put(task)

    # ------------------------------
    def running(self):
        """
        [ 运行 ]

        """

        # 开始一个监听循环
        while True:

            # 当 POOL运行时 任务队列为空 且 所有代理驱动都空闲时, 跳出监听循环
            if self._config_runtime.task_queue.qsize() == 0:
                if self._debug: LOG.info('任务消耗完毕, 正在等待任务执行完毕.')
                while True:
                    core_count = self._config_runtime._driver_queue.qsize()
                    if core_count == self._config_runtime.sign_driver_count:
                        if self._debug:
                            LOG.info('任务全部执行完毕, 退出.')
                        break
                break

            task = self._config_runtime.task_queue.get()
            driver = self._config_runtime.driver_queue.get()

            # 驱动冷启动时会占用大量的 CPU 资源, 为避免因为 CPU 占用率过高导致进程启动异常
            # 每隔 2 秒检查 CPU 占用率, 只有 CPU 占用率少于 50 时才会启动进程
            while True:
                if psutil.cpu_percent(2) < 50:
                    self._config_runtime.pool.apply_async(func=self._emit, args=[task, driver])
                    break

    # ------------------------------
    def _emit(self, task: Any, driver: Any):
        """
        [ 触发工作流 ]
        
        ---
        参数:
            task { Any } : 任务对象.
            driver { Any } : 驱动对象.

        """

        # 核心执行任务
        driver.start_task(task)

        # 核心执行完任务后, 放回核心队列等待
        self._config_runtime.driver_queue.put(driver)

        # 强制 GC 回收垃圾避免内存溢出
        gc.collect()



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.DB.Blueprint.SqlTask ]

"""

from rains.server.db.common import RainsDbParameterHandler

from rains.baseic.const import CONST
from rains.baseic.keyword import KEYWORD


# ----------------------------------
class SqlTask(object):
    """
    [ SQL语句 :: 任务相关 ]

    """

    @staticmethod
    def get_count_all() -> str:

        """
        [ 查询所有任务数量 ]
        
        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 
        
        SELECT COUNT({ KEYWORD.ARG.TID })
        
        FROM { KEYWORD.DB.TASKS_TABLE }
        
        """

    # ------------------------------
    @staticmethod
    def get_count_pass() -> str:
        """
        [ 查询全部通过的任务数量 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 
        
        SELECT COUNT({ KEYWORD.ARG.TID })

        FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.CASE_FAIL } == 0

        """

    # ------------------------------
    @staticmethod
    def get_count_fail() -> str:
        """
        [ 查询存在异常的任务数量 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.TID })

        FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.CASE_FAIL } != 0

        """

    # ------------------------------
    @staticmethod
    def get_count_from_data(paras: dict) -> str:
        """
        [ 查询指定日期中的任务数量 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.TID })

        FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.CREATED_DATE } = '{ paras[KEYWORD.ARG.CREATED_DATE] }'

        """

    # ------------------------------
    @staticmethod
    def get_date_list() -> str:
        """
        [ 查询去重后的所有存在任务的日期列表 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        SELECT DISTINCT { KEYWORD.ARG.CREATED_DATE } 

        FROM { KEYWORD.DB.TASKS_TABLE }

        ORDER BY ID DESC

        """

    # ------------------------------
    @staticmethod
    def get_spend_time_all() -> str:
        """
        [ 查询所有任务的消耗时间 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT SUM({ KEYWORD.ARG.SPEND_TIME_S })

        FROM { KEYWORD.DB.TASKS_TABLE }

        """

    # ------------------------------
    @staticmethod
    def get_spend_time_from_data(paras: dict) -> str:
        """
        [ 查询指定日期中所有任务的消耗时间 ]
        
        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT SUM({ KEYWORD.ARG.SPEND_TIME_S })

        FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.CREATED_DATE } = '{ paras[KEYWORD.ARG.CREATED_DATE] }'

        """

    # ------------------------------
    @staticmethod
    def get_info_all(
        page: int = 1,
        number: int = 10
    ) -> str:
        """
        [ 查询所有任务详情信息 ]

        ---
        参数:
            page { int } : 页数.
            number { int } : 单页数据量.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        SELECT * 

        FROM { KEYWORD.DB.TASKS_TABLE }

        ORDER BY { KEYWORD.ARG.TID } DESC 

        LIMIT { RainsDbParameterHandler.get_desc_limit(page, number) }

        """

    # ------------------------------
    @staticmethod
    def get_info_from_tid(paras: dict) -> str:
        """
        [ 查询指定TID的任务详情信息 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { int } : 任务编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        SELECT *

        FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.TID } = { paras[KEYWORD.ARG.TID] }

        """

    # ------------------------------
    @staticmethod
    def add(paras: dict) -> str:
        """
        [ 创建任务 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            name { str } : 任务名称.
            type { str } : 任务类型.
            remark { str } : 任务备注.
            created_date { str } : 创建日期.
            state { str } : 任务状态.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        INSERT INTO { KEYWORD.DB.TASKS_TABLE } (

             { KEYWORD.ARG.TID }, 
             { KEYWORD.ARG.NAME }, 
             { KEYWORD.ARG.TYPE }, 
             { KEYWORD.ARG.REMARK }, 
             { KEYWORD.ARG.CREATED_DATE }, 
             { KEYWORD.ARG.STATE }, 
             { KEYWORD.ARG.START_TIME }, 
             { KEYWORD.ARG.END_TIME }, 
             { KEYWORD.ARG.SPEND_TIME_S }, 
             { KEYWORD.ARG.CASE_ALL }, 
             { KEYWORD.ARG.CASE_PASS }, 
             { KEYWORD.ARG.CASE_FAIL }
        )

        VALUES (

            NULL, 
            '{ paras[KEYWORD.ARG.NAME] }', 
            '{ paras[KEYWORD.ARG.TYPE] }', 
            '{ paras[KEYWORD.ARG.REMARK] }', 
            '{ paras[KEYWORD.ARG.CREATED_DATE] }', 
            '{ paras[KEYWORD.ARG.STATE] }', 
            NULL, NULL, NULL, NULL, NULL, NULL
        )

        """

    # ------------------------------
    @staticmethod
    def update(paras: dict) -> str:
        """
        [ 更新任务 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { int } : 任务编号.
            state { str } : 任务状态.
            start_time { str } : 开始时间.
            end_time { str } : 结束时间.
            spend_time_s { int } : 消耗时间(秒).
            case_all { int } : 所有用例计数.
            case_pass { int } : 成功的用例计数.
            case_fail { int } : 失败用例数计数.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        UPDATE { KEYWORD.DB.TASKS_TABLE }

        SET
            { KEYWORD.ARG.STATE }  = '{ paras[KEYWORD.ARG.STATE] }', 
            { KEYWORD.ARG.START_TIME }    = '{ paras[KEYWORD.ARG.START_TIME] }', 
            { KEYWORD.ARG.END_TIME }      = '{ paras[KEYWORD.ARG.END_TIME] }', 
            { KEYWORD.ARG.SPEND_TIME_S }  = '{ paras[KEYWORD.ARG.SPEND_TIME_S] }', 
            { KEYWORD.ARG.CASE_ALL }      =  { paras[KEYWORD.ARG.CASE_ALL] }, 
            { KEYWORD.ARG.CASE_PASS }     =  { paras[KEYWORD.ARG.CASE_PASS] }, 
            { KEYWORD.ARG.CASE_FAIL }     =  { paras[KEYWORD.ARG.CASE_FAIL] }

        WHERE { KEYWORD.ARG.TID } = { paras[KEYWORD.ARG.TID] }

        """

    # ------------------------------
    @staticmethod
    def delete(paras: dict) -> str:
        """
        [ 删除任务 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { int } : 任务编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        DELETE FROM { KEYWORD.DB.TASKS_TABLE }

        WHERE { KEYWORD.ARG.TID } = { paras[KEYWORD.ARG.TID] }

        """

    # ------------------------------ 
    @staticmethod
    def delete_all() -> str:
        """
        [ 删除所有的任务 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        DELETE FROM { KEYWORD.DB.TASKS_TABLE }

        """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 工具相关 ]

"""

import re
import yaml

from rains.baseic.error import YamlHandlerError


# ----------------------------------
class Regular(object):
    """
    [ 正则封装 ]

    """

    # ------------------------------
    @staticmethod
    def get_function_doc_name(doc: str) -> str:
        """
        [ 获取函数注释中的名称 ]

        """
        
        return re.findall(r'\[ (.*) \]', doc)[0]

    # ------------------------------
    @staticmethod
    def get_function_doc_explain(doc: str) -> list:
        """
        [ 获取函数注释中的说明 ]

        """

        return re.findall(r'\* (.*)', doc)


# ----------------------------------
class YamlHandler(object):
    """
    [ YAML处理器 ]

    """

    # ------------------------------
    @staticmethod
    def read(yaml_path: str) -> dict:
        ...

    # ------------------------------
    @staticmethod
    def write(
        yaml_path: str, 
        data_dict: dict,
        mode: str = 'w',
        encoding: str = 'UTF-8'
    ) -> bool:
        """
        [ 写入 ]

        ---
        参数:
            yaml_path { str } : YAML文件路径.
            data_dict { dict } : 数据字典.
            mode { str } : 模式.
            encoding { str } : 编码.

        ---
        返回:
            bool : 成功写入返回 True, 否则抛出 YamlHandlerError 异常.

        """
        try:
            with open(yaml_path, mode, encoding=encoding) as f:
                yaml.dump(data_dict, f, allow_unicode=True)
            return True

        except BaseException as error:
            raise YamlHandlerError(error)



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 核心通用模块 ]

"""



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Common.Task.TaskFnAgency ]

"""

import inspect
from typing import Any

from rains.baseic.keyword import KEYWORD


# ----------------------------------
class TaskFnAgency(object):
    """
    [ 函数代理 ]

    * 该类主要职责于执行 JSON 译文串.

    * 通过传递模块名、类名、方法名、元素定位、传递参数的方式调用目标函数.

    """

    _kit_plant: Any = None
    """ [ 测试套件工厂构建类 ] """

    _module_living = None
    """ [ 模块类实例 ] """

    _class_living = None
    """ [ 执行类实例 ] """

    _translation_dict: dict
    """ [ JSON 译文串容器 ] """

    # ------------------------------
    def __init__(self, kit_plant: Any):
        """
        [ 函数代理 ]
        
        ---
        参数:
            kit_plant { Any } : 测试套件工厂构建类.

        """

        self._kit_plant = kit_plant

    # ------------------------------
    def map(self, translation_dict: dict):
        """
        [ 映射函数 ]

        * 通过传递类名、方法名、参数字典的形式调用指定函数.
        
        ---
        参数:
            translation_dict { dict } : JSON 串.

        """

        self._translation_dict = translation_dict
        self._analysis_map_living()
        self._analysis_run_living()
        return self._running_function()

    # ------------------------------
    def _analysis_map_living(self):
        """
        [ 解析映射类实例 ]

        """

        try:
            module = self._translation_dict[KEYWORD.TASK.JSON_ARG_MODULE]

            legal_modules = [m for m in type(self._kit_plant).__dict__.keys() if '_' not in m]
            if module not in legal_modules:
                err_mgs = f"异常 :: 模块不存在 :: '{ module }' not in { legal_modules } !"
                raise Exception(err_mgs)

            if module == 'element':

                if KEYWORD.TASK.JSON_ARG_ELEMENT not in self._translation_dict.keys():
                    err_mgs = f"缺少元素定位描述 :: 参数必须携带 [{ KEYWORD.TASK.JSON_ARG_ELEMENT }] !"
                    raise Exception(err_mgs)

                element: dict = self._translation_dict[KEYWORD.TASK.JSON_ARG_ELEMENT]
                element.update({'self': self._kit_plant})

                parameters = []
                sign = inspect.signature(self._kit_plant.element)
                for k, v in sign.parameters.items():
                    if v.default == inspect._empty:
                        if k not in element.keys():
                            raise Exception(f'元素定位描述缺少参数 :: [{ k }]')
                    value = v.default if k not in element.keys() else element[k]
                    parameters.append(value)

                self._map_living = self._kit_plant.element(*parameters)
            
            else:
                self._map_living = type(self._kit_plant).__dict__[f'_map_{ module }'](self._kit_plant)

        except BaseException as err:
            raise Exception(str(err))

    # ------------------------------
    def _analysis_run_living(self):
        """
        [ 解析执行类实例 ]

        """

        try:
            run_class = self._translation_dict[KEYWORD.TASK.JSON_ARG_CLASS]
            unstrained_class_dict = type(self._map_living).__dict__

            if run_class not in unstrained_class_dict:
                raise Exception(f'{ type(self._map_living) } there is no such map :: { run_class }')

            self._run_living = unstrained_class_dict[f'_map_{ run_class }'](self._map_living)
    
        except BaseException as err:
            raise Exception(str(err))

    # ------------------------------
    def _running_function(self):
        """
        [ 执行函数 ]

        """

        try:
            function = self._translation_dict[KEYWORD.TASK.JSON_ARG_FUNCTION]
            unstrained_class_dict = type(self._run_living).__dict__

            if function not in unstrained_class_dict:
                raise Exception(f'{ type(self._run_living) } there is no such function :: { function }')

            function_living = type(self._run_living).__dict__[function]

            parameter: dict = {}
            if KEYWORD.TASK.JSON_ARG_PARAMETER in self._translation_dict.keys():
                parameter = self._translation_dict[KEYWORD.TASK.JSON_ARG_PARAMETER]
            parameter.update({'self': self._run_living})

            parameters = []
            sign = inspect.signature(function_living)
            for k, v in sign.parameters.items():
                if v.default == inspect._empty:
                    if k not in parameter.keys():
                        raise Exception(f'{ function } missing para :: { k }')
                value = v.default if k not in parameter.keys() else parameter[k]
                parameters.append(value)

            return function_living(*parameters)
            
        except BaseException as err:
            raise Exception(str(err))



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
""" 
[ 全局关键字 ]

"""

from rains.baseic.keyword.keyword_arg import KeywordArg
from rains.baseic.keyword.keyword_db import KeywordDB
from rains.baseic.keyword.keyword_task import KeywordTask
from rains.baseic.keyword.keyword_config import KeywordConfig


# ----------------------------------
class Keyword(object):
    """ [ 全局关键字 ] """

    # ------------------------------
    @property
    def ARG(self) -> KeywordArg:
        """ [ 字段关键字 ] """
        return KeywordArg

    # ------------------------------
    @property
    def DB(self) -> KeywordDB:
        """ [ 数据库关键字 ] """
        return KeywordDB

    # ------------------------------
    @property
    def TASK(self) -> KeywordTask:
        """ [ 任务关键字 ] """
        return KeywordTask

    # ------------------------------
    @property
    def CONFIG(self) -> KeywordConfig:
        """ [ 配置关键字 ] """
        return KeywordConfig


KEYWORD: Keyword = Keyword()
""" [ 全局关键字 ] """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.Kit.Web.WebTask ]

"""

from rains.baseic.const import CONST
from rains.baseic.interface.i_task import ITask
from rains.kit.web.web_plant import WebPlant


# ------- WEB TASK -------
class WebTask(ITask):
    """
    [ Web 任务集 ]

    """

    type: str = CONST.TYPE.PROJECT_TYPE_WEB
    """ [ 任务类型 ] """

    plant: WebPlant
    """ [ 工厂对象 ] """

    # ------------------------------
    def set_task_init(self):
        """
        [ 设置任务初始化 ]

        * 该接口将在 [ 任务 ] 开始时执行, 全程只会执行一次.

        """
        ...

    # ------------------------------
    def set_task_quit(self):
        """
        [ 设置任务注销 ]

        * 该接口将在 [ 任务 ] 结束后执行, 全程只会执行一次.

        """
        ...

    # ------------------------------
    def set_case_init(self):
        """
        [ 设置用例初始化 ]

        * 该接口将在每次 [ 用例 ] 开始时执行.

        """
        ...

    # ------------------------------
    def set_case_quit(self):
        """
        [ 设置用例注销 ]

        * 该接口将在每次 [ 用例 ] 结束后执行.

        """
        ...



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 行为记录器 ]

"""

from rains.baseic.decorator import singleton_pattern


# ----------------------------------
@singleton_pattern
class ActionRecorder(object):
    """
    [ 行为记录器 ]

    * 用以记录一些希望后续被取出的行为信息, 通常用于多个类之间的数据交互.
    * 通过编号进行数据的写入与取出, 且取出后, 将会清空被取出编号的数据.

    """

    # 缓冲存储器
    _buffer: dict

    # ------------------------------
    def __init__(self):
        """
        [ 行为记录器 ]
 
        """

        self._buffer = {}

    # ------------------------------
    def write(self, id: int, record: str):
        """
        [ 录入信息 ]

        * 将编号的行为信息录入到缓冲存储器中.

        ---
        参数:
            id { int } : 编号.
            record { str } : 行为信息.

        """

        if id in self._buffer:
            self._buffer[id].append(record)

        else:
            self._buffer.update({id: [record]})

    # ------------------------------
    def take(self, id: int) -> list:
        """
        [ 取出信息 ]

        * 取出编号的行为信息, 清空缓冲存储器该编号的所有数据.

        ---
        参数:
            id { int } : 编号.

        ---
        返回:
            list : 核心的行为信息列表.

        """

        try:
            records = self._buffer[id]
            del self._buffer[id]
            return records
        
        except KeyError:
            return []


ACTION_RECORDER: ActionRecorder = ActionRecorder()
""" [ 行为记录器实例 ] """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 全局常量 ]

"""

from rains.baseic.const.const_api import ConstApi
from rains.baseic.const.const_state import ConstState
from rains.baseic.const.const_sys import ConstSys
from rains.baseic.const.const_type import ConstType


# ----------------------------------
class Const(object):
    """ [ 全局常量 ] """

    # ------------------------------
    @property
    def API(self) -> ConstApi:
        """ [ 接口常量 ] """
        return ConstApi

    # ------------------------------
    @property
    def STATE(self) -> ConstState:
        """ [ 状态常量 ] """
        return ConstState

    # ------------------------------
    @property
    def SYS(self) -> ConstSys:
        """ [ 系统常量 ] """
        return ConstSys

    # ------------------------------
    @property
    def TYPE(self) -> ConstType:
        """ [ 类型常量 ] """
        return ConstType


CONST: Const = Const()
""" [ 全局常量 ] """



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ Rains.DB.Blueprint.SqlCase ]

"""

from rains.server.db.common import RainsDbParameterHandler

from rains.baseic.const import CONST
from rains.baseic.keyword import KEYWORD


# ----------------------------------
class SqlCase(object):
    """
    [ SQL语句 :: 用例相关 ]

    """

    # ------------------------------
    @staticmethod
    def get_count_all() -> str:
        """
        [ 查询所有用例数量 ]
        
        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """
        
        return f"""

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE }

        """

    # ------------------------------
    @staticmethod
    def get_count_pass() -> str:
        """
        [ 查询所有通过的用例数量 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """
        
        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE }

        WHERE { KEYWORD.ARG.STATE } == '{ CONST.STATE.END }'

        """

    # ------------------------------
    @staticmethod
    def get_count_fail() -> str:
        """
        [ 查询所有非'成功'的用例数量 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """
        
        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE }

        WHERE { KEYWORD.ARG.STATE } != '{ CONST.STATE.END }'

        """

    # ------------------------------
    @staticmethod
    def get_count_from_date(paras: dict) -> str:
        """
        [ 查询指定日期里所有用例数量 ]
        
        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE }

        WHERE { KEYWORD.ARG.CREATED_DATE } == '{ paras[KEYWORD.ARG.CREATED_DATE] }'

        """

    # ------------------------------
    @staticmethod
    def get_count_fail_from_data(paras: dict) -> str:
        """
        [ 查询指定日期里所有非'成功'用例数量 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE }

        WHERE { KEYWORD.ARG.STATE } != '{ CONST.STATE.END }'

        AND { KEYWORD.ARG.CREATED_DATE } == '{ paras[KEYWORD.ARG.CREATED_DATE] }'

        """

    # ------------------------------
    @staticmethod
    def get_count_from_tid(paras: dict) -> str:
        """
        [ 查询指定任务下所有用例数量 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { int } : 所属任务编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE } 

        WHERE { KEYWORD.ARG.TID } = { paras[KEYWORD.ARG.TID] }

        """

    # ------------------------------
    @staticmethod
    def get_count_form_date(paras: dict) -> str:
        """
        [ 查询指定执行日期的用例数量 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        SELECT COUNT({ KEYWORD.ARG.CID }) FROM { KEYWORD.DB.CASES_TABLE } 

        WHERE { KEYWORD.ARG.CREATED_DATE } = '{ paras[KEYWORD.ARG.CREATED_DATE] }'

        """

    # ------------------------------
    @staticmethod
    def get_info_from_tid(paras: dict) -> str:
        """
        [ 查询任务所有用例详情信息列表 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { str } : 所属任务编号.
            
        ---
        参数字典非必要参数:
            state { str } : 状态.
            page { int } : 页数.
            number { int } : 单页数据量.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        if KEYWORD.ARG.STATE in paras.keys():
            state = f'{ KEYWORD.ARG.STATE } = "{ paras[KEYWORD.ARG.STATE] }"'
        else:
            state = f'{ KEYWORD.ARG.STATE } in ("{ CONST.STATE.RUN }", ' \
                    f'"{ CONST.STATE.END }", "{ CONST.STATE.ERR }")'

        return f"""

        SELECT * FROM { KEYWORD.DB.CASES_TABLE }

        WHERE { state } AND { KEYWORD.ARG.TID } = { paras[KEYWORD.ARG.TID] }

        ORDER BY ID DESC

        LIMIT { RainsDbParameterHandler.get_desc_limit(paras) }

        """

    # ------------------------------
    @staticmethod
    def get_info_from_cid(paras: dict) -> str:
        """
        [ 查询指定编号用例详情信息列表 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            cid { int } : 用例编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        SELECT * FROM { KEYWORD.DB.CASES_TABLE }

        WHERE id = { paras[KEYWORD.ARG.CID] }

        """

    # ------------------------------
    @staticmethod
    def add(paras: dict) -> str:
        """
        [ 创建用例 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            tid { int } : 所属任务编号.
            name { str } : 用例名称.
            remark { str } : 用例备注信息.
            created_date { str } : 创建日期.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f""" 

        INSERT INTO { KEYWORD.DB.CASES_TABLE } (

            { KEYWORD.ARG.CID }, 
            { KEYWORD.ARG.TID }, 
            { KEYWORD.ARG.NAME }, 
            { KEYWORD.ARG.REMARK }, 
            { KEYWORD.ARG.STATE }, 
            { KEYWORD.ARG.CREATED_DATE }, 
            { KEYWORD.ARG.START_TIME }, 
            { KEYWORD.ARG.END_TIME }, 
            { KEYWORD.ARG.SPEND_TIME_S }, 
            { KEYWORD.ARG.RUN_STEP }
        )

        VALUES (

            NULL, 
            '{ paras[KEYWORD.ARG.TID] }', 
            '{ paras[KEYWORD.ARG.NAME] }', 
            '{ paras[KEYWORD.ARG.REMARK] }', 
            '{ CONST.STATE.RUN }', 
            '{ paras[KEYWORD.ARG.CREATED_DATE] }', 
             NULL, 
             NULL, 
             NULL, 
             NULL
        )

        """

    # ------------------------------
    @staticmethod
    def update(paras: dict) -> str:
        """
        [ 更新用例 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            cid { int } : 用例编号.
            state { str } : 用例状态.
            start_time { str } : 开始时间.
            end_time { str } : 结束时间.
            spend_time_s { str } : 消耗时间(秒).
            run_step { str } : 运行步骤.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        UPDATE { KEYWORD.DB.CASES_TABLE }

        SET 
            { KEYWORD.ARG.STATE }         = '{ paras[KEYWORD.ARG.STATE] }', 
            { KEYWORD.ARG.START_TIME }    = '{ paras[KEYWORD.ARG.START_TIME] }', 
            { KEYWORD.ARG.END_TIME }      = '{ paras[KEYWORD.ARG.END_TIME] }', 
            { KEYWORD.ARG.SPEND_TIME_S }  = '{ paras[KEYWORD.ARG.SPEND_TIME_S] }', 
            { KEYWORD.ARG.RUN_STEP }      = '{ paras[KEYWORD.ARG.RUN_STEP] }'

        WHERE
            { KEYWORD.ARG.CID } = { paras[KEYWORD.ARG.CID] }

        """

    # ------------------------------
    @staticmethod
    def delete_from_cid(paras: dict) -> str:
        """
        [ 删除指定用例 ]
        
        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            cid { int } : 用例编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        DELETE FROM { KEYWORD.DB.CASES_TABLE }

        WHERE ID = { paras[KEYWORD.ARG.CID] }

        """

    # ------------------------------
    @staticmethod
    def delete_from_tid(paras: dict = None) -> str:
        """
        [ 删除指定任务的所有用例 ]

        ---
        参数:
            paras { dict } : 参数字典.
            
        ---
        参数字典必要参数:
            cid { int } : 用例编号.

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        DELETE FROM { KEYWORD.DB.CASES_TABLE }

        WHERE TID = { paras[KEYWORD.ARG.TID] }

        """

    # ------------------------------
    @staticmethod
    def delete_all() -> str:
        """
        [ 删除所有用例 ]

        ---
        返回:
            str : 拼接完成的 SQLite 语句.

        """

        return f"""

        DELETE FROM { KEYWORD.DB.CASES_TABLE }

        """




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from flask import Flask
from flask import jsonify

from rains.server.api.server_request_handler import ServerRequestHandler


app: Flask = Flask(__name__)


@app.route('/get1', methods=['GET'])
def get1():
    return jsonify({
        'code': 0,
        'data': 'get1'
    })


@app.route('/get2', methods=['GET'])
def get2():
    return jsonify({
        'code': 0,
        'data': 'get2'
    })


@app.route('/post1', methods=['POST'])
def post1():
    a = ServerRequestHandler.analysis_request_parameter(['v1', 'v2'], ['v1'])
    return jsonify({
        'code': 0,
        'data': a
    })
    
    
@app.route('/post2', methods=['POST'])
def post2():
    token = ServerRequestHandler.analysis_request_headers(['Authorization'], ['Authorization'])
    a = ServerRequestHandler.analysis_request_parameter(['v1', 'v2'], ['v1'])
    return jsonify({
        'code': 0,
        'data': a,
        'token': token['Authorization']
    })


app.run()



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 命令行工具实现 ]

"""

import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))


command: str = 'order'
if len(sys.argv) > 1:
    command: str = sys.argv[1]

# ------------------------------
if command in ['order', '-order', '-o']:
    from rains.external.order import function_order
    function_order()

# ------------------------------
elif command in ['init', '-init', '-i']:
    from rains.external.init_engineering import function_init_engineering
    function_init_engineering()

# ------------------------------
elif command in ['make', '-make', '-m']:
    from rains.external.create_project import function_create_project
    function_create_project()

# ------------------------------
elif command in ['task', '-task', '-t']:
    from rains.external.create_task import function_create_task
    function_create_task()

# ------------------------------
elif command in ['run', '-run', '-r']:
    from rains.external.run_task import function_run_task
    function_run_task()

# ------------------------------
elif command in ['server', '-server', '-s']:
    from rains.external.run_server import function_run_server
    function_run_server()

# ------------------------------
elif command in ['help', '-help', '-h']:
    from rains.external.help import function_help
    function_help()

# ------------------------------
elif command in ['version', '-version', '-v']:
    from rains.external.version import function_version
    function_version()

else:
    print('error:: rains不存在这样的指令!')



#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 错误类型 ]

"""

import re


# ----------------------------------
class _RainsBaseError(Exception):
    """ [ 自定义异常基类 ] """
    def __init__(self, error: BaseException):
        self.message = str(error)
        self.err_name = re.findall(r'\[ (.*) \]', self.__doc__)[0]
    def __str__(self) -> str:
        return ''.join([self.err_name, ' :: ', self.message])


# ----------------------------------
class CreationLogLivingError(_RainsBaseError):
    """ [ 创建日志实例异常 ] """


# ----------------------------------
class ServerAnalysisParameterError(_RainsBaseError):
    """ [ 服务端解析参数异常 ] """


# ----------------------------------
class ServerInspectionParameterError(_RainsBaseError):
    """ [ 服务端检查参数异常 ] """
    def __str__(self) -> str:
        return ''.join([self.err_name, ' :: ', f'参数 [{ self.message }] 是必须的 !'])


# ----------------------------------
class WebFindHandleFindElementError(_RainsBaseError):
    """ [ Web定位器获取元素异常 ] """


# ----------------------------------
class RainsDbInitError(_RainsBaseError):
    """ [ Rains 数据库初始化异常 ] """


# ----------------------------------
class RainsDbReadError(_RainsBaseError):
    """ [ Rains 数据库读取异常 ] """


# ----------------------------------
class RainsDbWriteError(_RainsBaseError):
    """ [ Rains 数据库写入异常 ] """


# ----------------------------------
class TaskInstructionError(Exception):
    """ [ 任务指令解析异常 ] """


# ----------------------------------
class YamlHandlerError(Exception):
    """ [ YAML处理器异常 ] """
    


#!/usr/bin/env python3
# coding=UTF-8
#
# Copyright 2022. quinn.7@foxmail.com All rights reserved.
# Author :: cat7
# Email  :: quinn.7@foxmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================
"""
[ 创建项目 ]

"""

import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.realpath(__file__)))))

from rains.baseic.environment.make import create_project


# ----------------------------------
def function_create_project():
    """
    [ 创建项目 ]

    """
    
    if len(sys.argv) >= 3:

        print('\n--------------------------------------\n')
        for p in sys.argv[2:]:
            project_name = ''.join([PROJECT_NAMING_BASE_PREFIX, p])
            print(f'项目 [{ project_name }] => ', end='')
            result = create_project(project_name)
            if result:
                print(_MESSAGE_CREATING_SUCCESSFUL)
            else:
                print(_MESSAGE_CREATING_UNSUCCESSFUL)
        print('\n--------------------------------------\n')

    else:
        print(_MESSAGE_ERROR)


# 项目命名前缀
PROJECT_NAMING_BASE_PREFIX: str = 'p_'

# 项目创建成功文案
_MESSAGE_CREATING_SUCCESSFUL: str = '创建成功!'

# 项目创建失败文案
_MESSAGE_CREATING_UNSUCCESSFUL: str = '创建失败, 项目已经存在!'

# 项目创建异常文案
_MESSAGE_ERROR: str = \
"""
--------------------------------------

指令键入异常!

正确格式:
>> rains -make [project_name]
>> rains -make [project_name] [project_name] ...

示例:
>> rains -make Project
>> rains -make Project Project2

--------------------------------------
"""
